#!/usr/bin/python

# Copyright (c) 2019, Nurfet Becirevic <nurfet.becirevic@gmail.com>
# Copyright (c) 2019, Tomas Karasek <tom.to.the.k@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: packet_project

short_description: Create/delete a project in Packet host

description:
  - Create/delete a project in Packet host.
  - API is documented at U(https://www.packet.com/developers/api/#projects).
version_added: '0.2.0'

author:
  - Tomas Karasek (@t0mk) <tom.to.the.k@gmail.com>
  - Nurfet Becirevic (@nurfet-becirevic) <nurfet.becirevic@gmail.com>

extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  state:
    description:
      - Indicate desired state of the target.
    default: present
    choices: ['present', 'absent']
    type: str

  payment_method:
    description:
      - Payment method is name of one of the payment methods available to your user.
      - When blank, the API assumes the default payment method.
    type: str

  auth_token:
    description:
      - Packet API token. You can also supply it in environment variable E(PACKET_API_TOKEN).
    type: str

  name:
    description:
      - Name for/of the project.
    type: str

  org_id:
    description:
      - UUID of the organization to create a project for.
      - When blank, the API assumes the default organization.
    type: str

  id:
    description:
      - UUID of the project which you want to remove.
    type: str

  custom_data:
    description:
      - Custom data about the project to create.
    type: str

requirements:
  - "packet-python >= 1.40"
"""

EXAMPLES = r"""
# All the examples assume that you have your Packet API token in env var PACKET_API_TOKEN.
# You can also pass the api token in module param auth_token.

- name: Create new project
  hosts: localhost
  tasks:
    - community.general.packet_project:
        name: "new project"

- name: Create new project within non-default organization
  hosts: localhost
  tasks:
    - community.general.packet_project:
        name: "my org project"
        org_id: a4cc87f9-e00f-48c2-9460-74aa60beb6b0

- name: Remove project by id
  hosts: localhost
  tasks:
    - community.general.packet_project:
        state: absent
        id: eef49903-7a09-4ca1-af67-4087c29ab5b6

- name: Create new project with non-default billing method
  hosts: localhost
  tasks:
    - community.general.packet_project:
        name: "newer project"
        payment_method: "the other visa"
"""

RETURN = r"""
name:
  description: Name of addressed project.
  type: str
  returned: success

id:
  description: UUID of addressed project.
  type: str
  returned: success
"""

from ansible.module_utils.basic import AnsibleModule, env_fallback

HAS_PACKET_SDK = True

try:
    import packet
except ImportError:
    HAS_PACKET_SDK = False

PACKET_API_TOKEN_ENV_VAR = "PACKET_API_TOKEN"


def act_on_project(target_state, module, packet_conn):
    result_dict = {"changed": False}
    given_id = module.params.get("id")
    given_name = module.params.get("name")
    if given_id:
        matching_projects = [p for p in packet_conn.list_projects() if given_id == p.id]
    else:
        matching_projects = [p for p in packet_conn.list_projects() if given_name == p.name]

    if target_state == "present":
        if len(matching_projects) == 0:
            org_id = module.params.get("org_id")
            custom_data = module.params.get("custom_data")
            payment_method = module.params.get("payment_method")

            if not org_id:
                params = {"name": given_name, "payment_method_id": payment_method, "customdata": custom_data}
                new_project_data = packet_conn.call_api("projects", "POST", params)
                new_project = packet.Project(new_project_data, packet_conn)
            else:
                new_project = packet_conn.create_organization_project(
                    org_id=org_id, name=given_name, payment_method_id=payment_method, customdata=custom_data
                )

            result_dict["changed"] = True
            matching_projects.append(new_project)

        result_dict["name"] = matching_projects[0].name
        result_dict["id"] = matching_projects[0].id
    else:
        if len(matching_projects) > 1:
            _msg = f"More than projects matched for module call with state = absent: {matching_projects}"
            module.fail_json(msg=_msg)

        if len(matching_projects) == 1:
            p = matching_projects[0]
            result_dict["name"] = p.name
            result_dict["id"] = p.id
            result_dict["changed"] = True
            try:
                p.delete()
            except Exception as e:
                _msg = f"while trying to remove project {p.name}, id {p.id}, got error: {e}"
                module.fail_json(msg=_msg)
    return result_dict


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(choices=["present", "absent"], default="present"),
            auth_token=dict(type="str", fallback=(env_fallback, [PACKET_API_TOKEN_ENV_VAR]), no_log=True),
            name=dict(type="str"),
            id=dict(type="str"),
            org_id=dict(type="str"),
            payment_method=dict(type="str"),
            custom_data=dict(type="str"),
        ),
        supports_check_mode=True,
        required_one_of=[
            (
                "name",
                "id",
            )
        ],
        mutually_exclusive=[
            ("name", "id"),
        ],
    )
    if not HAS_PACKET_SDK:
        module.fail_json(msg="packet required for this module")

    if not module.params.get("auth_token"):
        _fail_msg = f"if Packet API token is not in environment variable {PACKET_API_TOKEN_ENV_VAR}, the auth_token parameter is required"
        module.fail_json(msg=_fail_msg)

    auth_token = module.params.get("auth_token")

    packet_conn = packet.Manager(auth_token=auth_token)

    state = module.params.get("state")

    if state in ["present", "absent"]:
        if module.check_mode:
            module.exit_json(changed=False)

        try:
            module.exit_json(**act_on_project(state, module, packet_conn))
        except Exception as e:
            module.fail_json(msg=f"failed to set project state {state}: {e}")
    else:
        module.fail_json(msg=f"{state} is not a valid state for this module")


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Scaleway user data management module
#
# Copyright (C) 2018 Online SAS.
# https://www.scaleway.com
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: scaleway_user_data
short_description: Scaleway user_data management module
author: Remy Leone (@remyleone)
description:
  - This module manages user_data on compute instances on Scaleway.
  - It can be used to configure cloud-init for instance.
extends_documentation_fragment:
  - community.general._scaleway
  - community.general._attributes
  - community.general._scaleway.actiongroup_scaleway

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  action_group:
    version_added: 11.3.0

options:

  server_id:
    type: str
    description:
      - Scaleway Compute instance ID of the server.
    required: true

  user_data:
    type: dict
    description:
      - User defined data. Typically used with C(cloud-init).
      - Pass your C(cloud-init) script here as a string.

  region:
    type: str
    description:
      - Scaleway compute zone.
    required: true
    choices:
      - ams1
      - EMEA-NL-EVS
      - ams2
      - ams3
      - par1
      - EMEA-FR-PAR1
      - par2
      - EMEA-FR-PAR2
      - par3
      - waw1
      - EMEA-PL-WAW1
      - waw2
      - waw3
"""

EXAMPLES = r"""
- name: Update the cloud-init
  community.general.scaleway_user_data:
    server_id: '5a33b4ab-57dd-4eb6-8b0a-d95eb63492ce'
    region: ams1
    user_data:
      cloud-init: 'final_message: "Hello World!"'
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._scaleway import (
    SCALEWAY_LOCATION,
    Scaleway,
    scaleway_argument_spec,
)


def patch_user_data(compute_api, server_id, key, value):
    compute_api.module.debug("Starting patching user_data attributes")

    path = f"servers/{server_id}/user_data/{key}"
    response = compute_api.patch(path=path, data=value, headers={"Content-Type": "text/plain"})
    if not response.ok:
        msg = f"Error during user_data patching: {response.status_code} {response.body}"
        compute_api.module.fail_json(msg=msg)

    return response


def delete_user_data(compute_api, server_id, key):
    compute_api.module.debug(f"Starting deleting user_data attributes: {key}")

    response = compute_api.delete(path=f"servers/{server_id}/user_data/{key}")

    if not response.ok:
        msg = f"Error during user_data deleting: ({response.status_code}) {response.body}"
        compute_api.module.fail_json(msg=msg)

    return response


def get_user_data(compute_api, server_id, key):
    compute_api.module.debug("Starting patching user_data attributes")

    path = f"servers/{server_id}/user_data/{key}"
    response = compute_api.get(path=path)
    if not response.ok:
        msg = f"Error during user_data patching: {response.status_code} {response.body}"
        compute_api.module.fail_json(msg=msg)

    return response.json


def core(module):
    region = module.params["region"]
    server_id = module.params["server_id"]
    user_data = module.params["user_data"]
    changed = False

    module.params["api_url"] = SCALEWAY_LOCATION[region]["api_endpoint"]
    compute_api = Scaleway(module=module)

    user_data_list = compute_api.get(path=f"servers/{server_id}/user_data")
    if not user_data_list.ok:
        msg = f"Error during user_data fetching: {user_data_list.status_code} {user_data_list.body}"
        compute_api.module.fail_json(msg=msg)

    present_user_data_keys = user_data_list.json["user_data"]
    present_user_data = {
        key: get_user_data(compute_api=compute_api, server_id=server_id, key=key) for key in present_user_data_keys
    }

    if present_user_data == user_data:
        module.exit_json(changed=changed, msg=user_data_list.json)

    # First we remove keys that are not defined in the wished user_data
    for key in present_user_data:
        if key not in user_data:
            changed = True
            if compute_api.module.check_mode:
                module.exit_json(changed=changed, msg={"status": f"User-data of {server_id} would be patched."})

            delete_user_data(compute_api=compute_api, server_id=server_id, key=key)

    # Then we patch keys that are different
    for key, value in user_data.items():
        if key not in present_user_data or value != present_user_data[key]:
            changed = True
            if compute_api.module.check_mode:
                module.exit_json(changed=changed, msg={"status": f"User-data of {server_id} would be patched."})

            patch_user_data(compute_api=compute_api, server_id=server_id, key=key, value=value)

    module.exit_json(changed=changed, msg=user_data)


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(
        dict(
            region=dict(required=True, choices=list(SCALEWAY_LOCATION.keys())),
            user_data=dict(type="dict"),
            server_id=dict(required=True),
        )
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    core(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2020, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: shutdown
short_description: Shut down a machine
notes:
  - E(PATH) is ignored on the remote node when searching for the C(shutdown) command. Use O(search_paths) to specify locations
    to search if the default paths do not work.
  - The O(msg) and O(delay) options are not supported when a shutdown command is not found in O(search_paths), instead the
    module attempts to shutdown the system by calling C(systemctl shutdown).
description:
  - Shut downs a machine.
version_added: "1.1.0"
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.flow
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  action:
    support: full
  async:
    support: full
options:
  delay:
    description:
      - Seconds to wait before shutdown. Passed as a parameter to the shutdown command.
      - On Linux, macOS and OpenBSD, this is converted to minutes and rounded down. If less than 60, it is set to 0.
      - On Solaris and FreeBSD, this represents seconds.
    type: int
    default: 0
  msg:
    description:
      - Message to display to users before shutdown.
    type: str
    default: Shut down initiated by Ansible
  search_paths:
    description:
      - Paths to search on the remote machine for the C(shutdown) command.
      - I(Only) these paths are searched for the C(shutdown) command. E(PATH) is ignored in the remote node when searching
        for the C(shutdown) command.
    type: list
    elements: path
    default: ['/sbin', '/usr/sbin', '/usr/local/sbin']

seealso:
  - module: ansible.builtin.reboot
author:
  - Matt Davis (@nitzmahone)
  - Sam Doran (@samdoran)
  - Amin Vakil (@aminvakil)
"""

EXAMPLES = r"""
- name: Unconditionally shut down the machine with all defaults
  community.general.shutdown:

- name: Delay shutting down the remote node
  community.general.shutdown:
    delay: 60

- name: Shut down a machine with shutdown command in unusual place
  community.general.shutdown:
    search_paths:
      - '/lib/molly-guard'
"""

RETURN = r"""
shutdown:
  description: V(true) if the machine has been shut down.
  returned: always
  type: bool
  sample: true
"""



#!/usr/bin/python
# Copyright (c) 2018, Dusty Mabe <dusty@dustymabe.com>
# Copyright (c) 2018, Ansible Project
# Copyright (c) 2021, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: rpm_ostree_pkg
short_description: Install or uninstall overlay additional packages
version_added: "2.0.0"
description:
  - Install or uninstall overlay additional packages using C(rpm-ostree) command.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of overlay package to install or remove.
    required: true
    type: list
    elements: str
    aliases: [pkg]
  state:
    description:
      - State of the overlay package.
      - V(present) simply ensures that a desired package is installed.
      - V(absent) removes the specified package.
    choices: ['absent', 'present']
    default: 'present'
    type: str
  apply_live:
    description:
      - Adds the options C(--apply-live) when O(state=present).
      - Option is ignored when O(state=absent).
      - For more information, please see U(https://coreos.github.io/rpm-ostree/apply-live/).
    type: bool
    default: false
    version_added: 10.1.0
author:
  - Dusty Mabe (@dustymabe)
  - Abhijeet Kasurde (@Akasurde)
"""

EXAMPLES = r"""
- name: Install overlay package
  community.general.rpm_ostree_pkg:
    name: nfs-utils
    state: present

- name: Remove overlay package
  community.general.rpm_ostree_pkg:
    name: nfs-utils
    state: absent

- name: Apply the overlay package live
  community.general.rpm_ostree_pkg:
    name: nfs-utils
    state: present
    apply_live: true

# In case a different transaction is currently running the module would fail.
# Adding a delay can help mitigate this problem:
- name: Install overlay package
  community.general.rpm_ostree_pkg:
    name: nfs-utils
    state: present
  register: rpm_ostree_pkg
  until: rpm_ostree_pkg is not failed
  retries: 10
  delay: 30
"""

RETURN = r"""
action:
  description: Action performed.
  returned: always
  type: str
  sample: 'install'
packages:
  description: A list of packages specified.
  returned: always
  type: list
  sample: ["nfs-utils"]
cmd:
  description: Full command used for performed action.
  returned: always
  type: str
  sample: 'rpm-ostree uninstall --allow-inactive --idempotent --unchanged-exit-77 nfs-utils'
needs_reboot:
  description: Determine if machine needs a reboot to apply current changes.
  returned: success
  type: bool
  sample: true
  version_added: 10.1.0
"""

from ansible.module_utils.basic import AnsibleModule


class RpmOstreePkg:
    def __init__(self, module):
        self.module = module
        self.params = module.params
        self.state = module.params["state"]

    def ensure(self):
        results = dict(
            rc=0,
            changed=False,
            action="",
            packages=[],
            stdout="",
            stderr="",
            cmd="",
            needs_reboot=False,
        )

        # Ensure rpm-ostree command exists
        cmd = [self.module.get_bin_path("rpm-ostree", required=True)]

        # Decide action to perform
        if self.state == "present":
            results["action"] = "install"
            cmd.append("install")
        elif self.state == "absent":
            results["action"] = "uninstall"
            cmd.append("uninstall")

        # Add the options to the command line
        if self.params["apply_live"] and self.state == "present":
            cmd.extend(["--apply-live", "--assumeyes"])

        # Additional parameters
        cmd.extend(["--allow-inactive", "--idempotent", "--unchanged-exit-77"])
        for pkg in self.params["name"]:
            cmd.append(pkg)
            results["packages"].append(pkg)

        rc, out, err = self.module.run_command(cmd)

        # Determine if system needs a reboot to apply change
        if 'Changes queued for next boot. Run "systemctl reboot" to start a reboot' in out:
            results["needs_reboot"] = True

        results.update(
            dict(
                rc=rc,
                cmd=" ".join(cmd),
                stdout=out,
                stderr=err,
            )
        )

        # A few possible options:
        #     - rc=0  - succeeded in making a change
        #     - rc=77 - no change was needed
        #     - rc=?  - error
        if rc == 0:
            results["changed"] = True
        elif rc == 77:
            results["changed"] = False
            results["rc"] = 0
        else:
            self.module.fail_json(msg="non-zero return code", **results)

        self.module.exit_json(**results)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(default="present", choices=["absent", "present"]),
            name=dict(
                aliases=["pkg"],
                required=True,
                type="list",
                elements="str",
            ),
            apply_live=dict(
                type="bool",
                default=False,
            ),
        ),
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    rpm_ostree_pkg = RpmOstreePkg(module)
    rpm_ostree_pkg.ensure()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019, Nurfet Becirevic <nurfet.becirevic@gmail.com>
# Copyright (c) 2017, Tomas Karasek <tom.to.the.k@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: packet_volume

short_description: Create/delete a volume in Packet host

description:
  - Create/delete a volume in Packet host.
  - API is documented at U(https://www.packet.com/developers/api/#volumes).
version_added: '0.2.0'

author:
  - Tomas Karasek (@t0mk) <tom.to.the.k@gmail.com>
  - Nurfet Becirevic (@nurfet-becirevic) <nurfet.becirevic@gmail.com>

extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  state:
    description:
      - Desired state of the volume.
    default: present
    choices: ['present', 'absent']
    type: str

  project_id:
    description:
      - ID of project of the device.
    required: true
    type: str

  auth_token:
    description:
      - Packet API token. You can also supply it in environment variable E(PACKET_API_TOKEN).
    type: str

  name:
    description:
      - Selector for API-generated name of the volume.
    type: str

  description:
    description:
      - User-defined description attribute for Packet volume.
      - It is used used as idempotent identifier - if volume with given description exists, new one is not created.
    type: str

  id:
    description:
      - UUID of a volume.
    type: str

  plan:
    description:
      - V(storage_1) for standard tier, V(storage_2) for premium (performance) tier.
      - Tiers are described at U(https://www.packet.com/cloud/storage/).
    choices: ['storage_1', 'storage_2']
    default: 'storage_1'
    type: str

  facility:
    description:
      - Location of the volume.
      - Volumes can only be attached to device in the same location.
    type: str

  size:
    description:
      - Size of the volume in gigabytes.
    type: int

  locked:
    description:
      - Create new volume locked.
    type: bool
    default: false

  billing_cycle:
    description:
      - Billing cycle for new volume.
    choices: ['hourly', 'monthly']
    default: 'hourly'
    type: str

  snapshot_policy:
    description:
      - Snapshot policy for new volume.
    type: dict

    suboptions:
      snapshot_count:
        description:
          - How many snapshots to keep, a positive integer.
        required: true
        type: int

      snapshot_frequency:
        description:
          - Frequency of snapshots.
        required: true
        choices: ["15min", "1hour", "1day", "1week", "1month", "1year"]
        type: str

requirements:
  - "packet-python >= 1.35"
"""

EXAMPLES = r"""
# All the examples assume that you have your Packet API token in env var PACKET_API_TOKEN.
# You can also pass the api token in module param auth_token.

- hosts: localhost
  vars:
    volname: testvol123
    project_id: 53000fb2-ee46-4673-93a8-de2c2bdba33b

  tasks:
    - name: Create volume
      community.general.packet_volume:
        description: "{{ volname }}"
        project_id: "{{ project_id }}"
        facility: 'ewr1'
        plan: 'storage_1'
        state: present
        size: 10
        snapshot_policy:
          snapshot_count: 10
          snapshot_frequency: 1day
      register: result_create

    - name: Delete volume
      community.general.packet_volume:
        id: "{{ result_create.id }}"
        project_id: "{{ project_id }}"
        state: absent
"""

RETURN = r"""
id:
  description: UUID of specified volume.
  type: str
  returned: success
  sample: 53000fb2-ee46-4673-93a8-de2c2bdba33c
name:
  description: The API-generated name of the volume resource.
  type: str
  returned: if volume is attached/detached to/from some device
  sample: "volume-a91dc506"
description:
  description: The user-defined description of the volume resource.
  type: str
  returned: success
  sample: "Just another volume"
"""

import uuid

from ansible.module_utils.basic import AnsibleModule, env_fallback

HAS_PACKET_SDK = True


try:
    import packet
except ImportError:
    HAS_PACKET_SDK = False


PACKET_API_TOKEN_ENV_VAR = "PACKET_API_TOKEN"

VOLUME_PLANS = ["storage_1", "storage_2"]
VOLUME_STATES = ["present", "absent"]
BILLING = ["hourly", "monthly"]


def is_valid_uuid(myuuid):
    try:
        val = uuid.UUID(myuuid, version=4)
    except ValueError:
        return False
    return str(val) == myuuid


def get_volume_selector(module):
    if module.params.get("id"):
        i = module.params.get("id")
        if not is_valid_uuid(i):
            raise Exception(f"Volume ID '{i}' is not a valid UUID")
        return lambda v: v["id"] == i
    elif module.params.get("name"):
        n = module.params.get("name")
        return lambda v: v["name"] == n
    elif module.params.get("description"):
        d = module.params.get("description")
        return lambda v: v["description"] == d


def get_or_fail(params, key):
    item = params.get(key)
    if item is None:
        raise Exception(f"{key} must be specified for new volume")
    return item


def act_on_volume(target_state, module, packet_conn):
    return_dict = {"changed": False}
    s = get_volume_selector(module)
    project_id = module.params.get("project_id")
    api_method = f"projects/{project_id}/storage"
    all_volumes = packet_conn.call_api(api_method, "GET")["volumes"]
    matching_volumes = [v for v in all_volumes if s(v)]

    if target_state == "present":
        if len(matching_volumes) == 0:
            params = {
                "description": get_or_fail(module.params, "description"),
                "size": get_or_fail(module.params, "size"),
                "plan": get_or_fail(module.params, "plan"),
                "facility": get_or_fail(module.params, "facility"),
                "locked": get_or_fail(module.params, "locked"),
                "billing_cycle": get_or_fail(module.params, "billing_cycle"),
                "snapshot_policies": module.params.get("snapshot_policy"),
            }

            new_volume_data = packet_conn.call_api(api_method, "POST", params)
            return_dict["changed"] = True
            for k in ["id", "name", "description"]:
                return_dict[k] = new_volume_data[k]

        else:
            for k in ["id", "name", "description"]:
                return_dict[k] = matching_volumes[0][k]

    else:
        if len(matching_volumes) > 1:
            _msg = f"More than one volume matches in module call for absent state: {matching_volumes}"
            module.fail_json(msg=_msg)

        if len(matching_volumes) == 1:
            volume = matching_volumes[0]
            packet_conn.call_api(f"storage/{volume['id']}", "DELETE")
            return_dict["changed"] = True
            for k in ["id", "name", "description"]:
                return_dict[k] = volume[k]

    return return_dict


def main():
    module = AnsibleModule(
        argument_spec=dict(
            id=dict(type="str"),
            description=dict(type="str"),
            name=dict(type="str"),
            state=dict(choices=VOLUME_STATES, default="present"),
            auth_token=dict(type="str", fallback=(env_fallback, [PACKET_API_TOKEN_ENV_VAR]), no_log=True),
            project_id=dict(required=True),
            plan=dict(choices=VOLUME_PLANS, default="storage_1"),
            facility=dict(type="str"),
            size=dict(type="int"),
            locked=dict(type="bool", default=False),
            snapshot_policy=dict(type="dict"),
            billing_cycle=dict(type="str", choices=BILLING, default="hourly"),
        ),
        supports_check_mode=True,
        required_one_of=[("name", "id", "description")],
        mutually_exclusive=[
            ("name", "id"),
            ("id", "description"),
            ("name", "description"),
        ],
    )

    if not HAS_PACKET_SDK:
        module.fail_json(msg="packet required for this module")

    if not module.params.get("auth_token"):
        _fail_msg = f"if Packet API token is not in environment variable {PACKET_API_TOKEN_ENV_VAR}, the auth_token parameter is required"
        module.fail_json(msg=_fail_msg)

    auth_token = module.params.get("auth_token")

    packet_conn = packet.Manager(auth_token=auth_token)

    state = module.params.get("state")

    if state in VOLUME_STATES:
        if module.check_mode:
            module.exit_json(changed=False)

        try:
            module.exit_json(**act_on_volume(state, module, packet_conn))
        except Exception as e:
            module.fail_json(msg=f"failed to set volume state {state}: {e}")
    else:
        module.fail_json(msg=f"{state} is not a valid state for this module")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright 2016 Tomas Karasek <tom.to.the.k@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: packet_sshkey
short_description: Create/delete an SSH key in Packet host
description:
  - Create/delete an SSH key in Packet host.
  - API is documented at U(https://www.packet.net/help/api/#page:ssh-keys,header:ssh-keys-ssh-keys-post).
author: "Tomas Karasek (@t0mk) <tom.to.the.k@gmail.com>"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    description:
      - Indicate desired state of the target.
    default: present
    choices: ['present', 'absent']
    type: str
  auth_token:
    description:
      - Packet API token. You can also supply it in environment variable E(PACKET_API_TOKEN).
    type: str
  label:
    description:
      - Label for the key. If you keep it empty, it is read from key string.
    type: str
    aliases: [name]
  id:
    description:
      - UUID of the key which you want to remove.
    type: str
  fingerprint:
    description:
      - Fingerprint of the key which you want to remove.
    type: str
  key:
    description:
      - Public Key string (V({type} {base64 encoded key} {description})).
    type: str
  key_file:
    description:
      - File with the public key.
    type: path

requirements:
  - packet-python
"""

EXAMPLES = r"""
# All the examples assume that you have your Packet API token in env var PACKET_API_TOKEN.
# You can also pass the api token in module param auth_token.

- name: Create sshkey from string
  hosts: localhost
  tasks:
    - community.general.packet_sshkey:
        key: "{{ lookup('file', 'my_packet_sshkey.pub') }}"

- name: Create sshkey from file
  hosts: localhost
  tasks:
    - community.general.packet_sshkey:
        label: key from file
        key_file: ~/ff.pub

- name: Remove sshkey by id
  hosts: localhost
  tasks:
    - community.general.packet_sshkey:
        state: absent
        id: eef49903-7a09-4ca1-af67-4087c29ab5b6
"""

RETURN = r"""
sshkeys:
  description: Information about sshkeys that were created/removed.
  type: list
  sample:
    [
      {
        "fingerprint": "5c:93:74:7c:ed:07:17:62:28:75:79:23:d6:08:93:46",
        "id": "41d61bd8-3342-428b-a09c-e67bdd18a9b7",
        "key": "ssh-dss AAAAB3NzaC1kc3MAAACBA ... MdDxfmcsCslJKgoRKSmQpCwXQtN2g== user@server",
        "label": "mynewkey33"
      }
    ]
  returned: always
"""

import os
import uuid

from ansible.module_utils.basic import AnsibleModule

HAS_PACKET_SDK = True
try:
    import packet
except ImportError:
    HAS_PACKET_SDK = False


PACKET_API_TOKEN_ENV_VAR = "PACKET_API_TOKEN"


def serialize_sshkey(sshkey):
    sshkey_data = {}
    copy_keys = ["id", "key", "label", "fingerprint"]
    for name in copy_keys:
        sshkey_data[name] = getattr(sshkey, name)
    return sshkey_data


def is_valid_uuid(myuuid):
    try:
        val = uuid.UUID(myuuid, version=4)
    except ValueError:
        return False
    return str(val) == myuuid


def load_key_string(key_str):
    ret_dict = {}
    key_str = key_str.strip()
    ret_dict["key"] = key_str
    cut_key = key_str.split()
    if len(cut_key) in [2, 3]:
        if len(cut_key) == 3:
            ret_dict["label"] = cut_key[2]
    else:
        raise Exception(f"Public key {key_str} is in wrong format")
    return ret_dict


def get_sshkey_selector(module):
    key_id = module.params.get("id")
    if key_id:
        if not is_valid_uuid(key_id):
            raise Exception(f"sshkey ID {key_id} is not valid UUID")
    selecting_fields = ["label", "fingerprint", "id", "key"]
    select_dict = {}
    for f in selecting_fields:
        if module.params.get(f) is not None:
            select_dict[f] = module.params.get(f)

    if module.params.get("key_file"):
        with open(module.params.get("key_file")) as _file:
            loaded_key = load_key_string(_file.read())
        select_dict["key"] = loaded_key["key"]
        if module.params.get("label") is None:
            if loaded_key.get("label"):
                select_dict["label"] = loaded_key["label"]

    def selector(k):
        if "key" in select_dict:
            # if key string is specified, compare only the key strings
            return k.key == select_dict["key"]
        else:
            # if key string not specified, all the fields must match
            return all(select_dict[f] == getattr(k, f) for f in select_dict)

    return selector


def act_on_sshkeys(target_state, module, packet_conn):
    selector = get_sshkey_selector(module)
    existing_sshkeys = packet_conn.list_ssh_keys()
    matching_sshkeys = list(filter(selector, existing_sshkeys))
    changed = False
    if target_state == "present":
        if matching_sshkeys == []:
            # there is no key matching the fields from module call
            # => create the key, label and
            newkey = {}
            if module.params.get("key_file"):
                with open(module.params.get("key_file")) as f:
                    newkey = load_key_string(f.read())
            if module.params.get("key"):
                newkey = load_key_string(module.params.get("key"))
            if module.params.get("label"):
                newkey["label"] = module.params.get("label")
            for param in ("label", "key"):
                if param not in newkey:
                    _msg = (
                        "If you want to ensure a key is present, you must "
                        "supply both a label and a key string, either in "
                        f"module params, or in a key file. {param} is missing"
                    )
                    raise Exception(_msg)
            matching_sshkeys = []
            new_key_response = packet_conn.create_ssh_key(newkey["label"], newkey["key"])
            changed = True

            matching_sshkeys.append(new_key_response)
    else:
        # state is 'absent' => delete matching keys
        for k in matching_sshkeys:
            try:
                k.delete()
                changed = True
            except Exception as e:
                _msg = f"while trying to remove sshkey {k.label}, id {k.id} {target_state}, got error: {e}"
                raise Exception(_msg) from e

    return {"changed": changed, "sshkeys": [serialize_sshkey(k) for k in matching_sshkeys]}


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(choices=["present", "absent"], default="present"),
            auth_token=dict(default=os.environ.get(PACKET_API_TOKEN_ENV_VAR), no_log=True),
            label=dict(type="str", aliases=["name"]),
            id=dict(type="str"),
            fingerprint=dict(type="str"),
            key=dict(type="str", no_log=True),
            key_file=dict(type="path"),
        ),
        mutually_exclusive=[
            ("label", "id"),
            ("label", "fingerprint"),
            ("id", "fingerprint"),
            ("key", "fingerprint"),
            ("key", "id"),
            ("key_file", "key"),
        ],
    )

    if not HAS_PACKET_SDK:
        module.fail_json(msg="packet required for this module")

    if not module.params.get("auth_token"):
        _fail_msg = f"if Packet API token is not in environment variable {PACKET_API_TOKEN_ENV_VAR}, the auth_token parameter is required"
        module.fail_json(msg=_fail_msg)

    auth_token = module.params.get("auth_token")

    packet_conn = packet.Manager(auth_token=auth_token)

    state = module.params.get("state")

    if state in ["present", "absent"]:
        try:
            module.exit_json(**act_on_sshkeys(state, module, packet_conn))
        except Exception as e:
            module.fail_json(msg=f"failed to set sshkey state: {e}")
    else:
        module.fail_json(msg=f"{state} is not a valid state for this module")


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2013 Shaun Zinck <shaun.zinck at gmail.com>
# Copyright (c) 2015 Lawrence Leonard Gilbert <larry@L2G.to>
# Copyright (c) 2016 Jasper Lievisse Adriaanse <j at jasper.la>
#
# Written by Shaun Zinck
# Based on pacman module written by Afterburn <http://github.com/afterburn>
#  that was based on apt module written by Matthew Williams <matthew@flowroute.com>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pkgin
short_description: Package manager for SmartOS, NetBSD, et al
description:
  - 'The standard package manager for SmartOS, but also usable on NetBSD or any OS that uses C(pkgsrc). (Home: U(http://pkgin.net/)).'
author:
  - "Larry Gilbert (@L2G)"
  - "Shaun Zinck (@szinck)"
  - "Jasper Lievisse Adriaanse (@jasperla)"
notes:
  - 'Known bug with pkgin < 0.8.0: if a package is removed and another package depends on it, the other package is silently
    removed as well.'
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of package to install/remove;
      - Multiple names may be given, separated by commas.
    aliases: [pkg]
    type: list
    elements: str
  state:
    description:
      - Intended state of the package.
    choices: ['present', 'absent']
    default: present
    type: str
  update_cache:
    description:
      - Update repository database. Can be run with other steps or on its own.
    type: bool
    default: false
  upgrade:
    description:
      - Upgrade main packages to their newer versions.
    type: bool
    default: false
  full_upgrade:
    description:
      - Upgrade all packages to their newer versions.
    type: bool
    default: false
  clean:
    description:
      - Clean packages cache.
    type: bool
    default: false
  force:
    description:
      - Force package reinstall.
    type: bool
    default: false
"""

EXAMPLES = r"""
- name: Install package foo
  community.general.pkgin:
    name: foo
    state: present

- name: Install specific version of foo package
  community.general.pkgin:
    name: foo-2.0.1
    state: present

- name: Update cache and install foo package
  community.general.pkgin:
    name: foo
    update_cache: true

- name: Remove package foo
  community.general.pkgin:
    name: foo
    state: absent

- name: Remove packages foo and bar
  community.general.pkgin:
    name: foo,bar
    state: absent

- name: Update repositories as a separate step
  community.general.pkgin:
    update_cache: true

- name: Upgrade main packages (equivalent to pkgin upgrade)
  community.general.pkgin:
    upgrade: true

- name: Upgrade all packages (equivalent to pkgin full-upgrade)
  community.general.pkgin:
    full_upgrade: true

- name: Force-upgrade all packages (equivalent to pkgin -F full-upgrade)
  community.general.pkgin:
    full_upgrade: true
    force: true

- name: Clean packages cache (equivalent to pkgin clean)
  community.general.pkgin:
    clean: true
"""


import re

from ansible.module_utils.basic import AnsibleModule


class PackageState:
    PRESENT = 1
    NOT_INSTALLED = 2
    OUTDATED = 4
    NOT_FOUND = 8


def query_package(module, name):
    """Search for the package by name and return state of the package."""

    # test whether '-p' (parsable) flag is supported.
    rc, out, err = module.run_command([PKGIN_PATH, "-p", "-v"])

    if rc == 0:
        pflag = ["-p"]
        splitchar = ";"
    else:
        pflag = []
        splitchar = " "

    # Use "pkgin search" to find the package. The regular expression will
    # only match on the complete name.
    rc, out, err = module.run_command([PKGIN_PATH] + pflag + ["search", f"^{name}$"])

    # rc will not be 0 unless the search was a success
    if rc == 0:
        # Search results may contain more than one line (e.g., 'emacs'), so iterate
        # through each line to see if we have a match.
        packages = out.split("\n")

        for package in packages:
            # Break up line at spaces.  The first part will be the package with its
            # version (e.g. 'gcc47-libs-4.7.2nb4'), and the second will be the state
            # of the package:
            #     ''  - not installed
            #     '<' - installed but out of date
            #     '=' - installed and up to date
            #     '>' - installed but newer than the repository version

            if (
                package
                in ("reading local summary...", "processing local summary...", "downloading pkg_summary.xz done.")
            ) or (package.startswith("processing remote summary (")):
                continue

            pkgname_with_version, raw_state = package.split(splitchar)[0:2]

            # Search for package, stripping version
            # (results in sth like 'gcc47-libs' or 'emacs24-nox11')
            pkg_search_obj = re.search(r"^(.*?)\-[0-9][0-9.]*(nb[0-9]+)*", pkgname_with_version, re.M)

            # Do not proceed unless we have a match
            if not pkg_search_obj:
                continue

            # Grab matched string
            pkgname_without_version = pkg_search_obj.group(1)

            if name not in (pkgname_with_version, pkgname_without_version):
                continue

            # The package was found; now return its state
            if raw_state == "<":
                return PackageState.OUTDATED
            elif raw_state == "=" or raw_state == ">":
                return PackageState.PRESENT
            else:
                # Package found but not installed
                return PackageState.NOT_INSTALLED
            # no fall-through

        # No packages were matched
        return PackageState.NOT_FOUND

    # Search failed
    return PackageState.NOT_FOUND


def format_action_message(module, action, count):
    vars = {"actioned": action, "count": count}

    if module.check_mode:
        message = f"would have {vars['actioned']} {vars['count']} package"
    else:
        message = f"{vars['actioned']} {vars['count']} package"

    if count == 1:
        return message
    else:
        return f"{message}s"


def format_pkgin_command(module, command, package=None):
    # Not all commands take a package argument, so cover this up by passing
    # an empty string. Some commands (e.g. 'update') will ignore extra
    # arguments, however this behaviour cannot be relied on for others.
    if package is None:
        packages = []
    else:
        packages = [package]

    if module.params["force"]:
        force = ["-F"]
    else:
        force = []

    if module.check_mode:
        return [PKGIN_PATH, "-n", command] + packages
    else:
        return [PKGIN_PATH, "-y"] + force + [command] + packages


def remove_packages(module, packages):
    remove_c = 0

    # Using a for loop in case of error, we can report the package that failed
    for package in packages:
        # Query the package first, to see if we even need to remove
        if query_package(module, package) in [PackageState.NOT_INSTALLED, PackageState.NOT_FOUND]:
            continue

        rc, out, err = module.run_command(format_pkgin_command(module, "remove", package))

        if not module.check_mode and query_package(module, package) in [PackageState.PRESENT, PackageState.OUTDATED]:
            module.fail_json(msg=f"failed to remove {package}: {out}", stdout=out, stderr=err)

        remove_c += 1

    if remove_c > 0:
        module.exit_json(changed=True, msg=format_action_message(module, "removed", remove_c))

    module.exit_json(changed=False, msg="package(s) already absent")


def install_packages(module, packages):
    install_c = 0

    for package in packages:
        query_result = query_package(module, package)
        if query_result in [PackageState.PRESENT, PackageState.OUTDATED]:
            continue
        elif query_result is PackageState.NOT_FOUND:
            module.fail_json(msg=f"failed to find package {package} for installation")

        rc, out, err = module.run_command(format_pkgin_command(module, "install", package))

        if not module.check_mode and query_package(module, package) not in [
            PackageState.PRESENT,
            PackageState.OUTDATED,
        ]:
            module.fail_json(msg=f"failed to install {package}: {out}", stdout=out, stderr=err)

        install_c += 1

    if install_c > 0:
        module.exit_json(
            changed=True, msg=format_action_message(module, "installed", install_c), stdout=out, stderr=err
        )

    module.exit_json(changed=False, msg="package(s) already present")


def update_package_db(module):
    rc, out, err = module.run_command(format_pkgin_command(module, "update"))

    if rc == 0:
        if re.search("database for.*is up-to-date\n$", out):
            return False, "database is up-to-date"
        else:
            return True, "updated repository database"
    else:
        module.fail_json(msg="could not update package db", stdout=out, stderr=err)


def do_upgrade_packages(module, full=False):
    if full:
        cmd = "full-upgrade"
    else:
        cmd = "upgrade"

    rc, out, err = module.run_command(format_pkgin_command(module, cmd))

    if rc == 0:
        if re.search("^(.*\n|)nothing to do.\n$", out):
            module.exit_json(changed=False, msg="nothing left to upgrade")
    else:
        module.fail_json(msg=f"could not {cmd} packages", stdout=out, stderr=err)


def upgrade_packages(module):
    do_upgrade_packages(module)


def full_upgrade_packages(module):
    do_upgrade_packages(module, True)


def clean_cache(module):
    rc, out, err = module.run_command(format_pkgin_command(module, "clean"))

    if rc == 0:
        # There's no indication if 'clean' actually removed anything,
        # so assume it did.
        module.exit_json(changed=True, msg="cleaned caches")
    else:
        module.fail_json(msg="could not clean package cache", stdout=out, stderr=err)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(default="present", choices=["present", "absent"]),
            name=dict(aliases=["pkg"], type="list", elements="str"),
            update_cache=dict(default=False, type="bool"),
            upgrade=dict(default=False, type="bool"),
            full_upgrade=dict(default=False, type="bool"),
            clean=dict(default=False, type="bool"),
            force=dict(default=False, type="bool"),
        ),
        required_one_of=[["name", "update_cache", "upgrade", "full_upgrade", "clean"]],
        supports_check_mode=True,
    )

    global PKGIN_PATH
    PKGIN_PATH = module.get_bin_path("pkgin", True, ["/opt/local/bin"])

    module.run_command_environ_update = dict(LANGUAGE="C", LC_ALL="C")

    p = module.params

    if p["update_cache"]:
        c, msg = update_package_db(module)
        if not (p["name"] or p["upgrade"] or p["full_upgrade"]):
            module.exit_json(changed=c, msg=msg)

    if p["upgrade"]:
        upgrade_packages(module)
        if not p["name"]:
            module.exit_json(changed=True, msg="upgraded packages")

    if p["full_upgrade"]:
        full_upgrade_packages(module)
        if not p["name"]:
            module.exit_json(changed=True, msg="upgraded all packages")

    if p["clean"]:
        clean_cache(module)
        if not p["name"]:
            module.exit_json(changed=True, msg="cleaned caches")

    pkgs = p["name"]

    if p["state"] == "present":
        install_packages(module, pkgs)

    elif p["state"] == "absent":
        remove_packages(module, pkgs)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016-2023, Vlad Glagolev <scm@vaygr.net>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: simpleinit_msb
short_description: Manage services on Source Mage GNU/Linux
version_added: 7.5.0
description:
  - Controls services on remote hosts using C(simpleinit-msb).
author: "Vlad Glagolev (@vaygr)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - Name of the service.
    required: true
    aliases: ['service']
  state:
    type: str
    choices: [running, started, stopped, restarted, reloaded]
    description:
      - V(started)/V(stopped) are idempotent actions that do not run commands unless necessary. V(restarted) always bounces
        the service. V(reloaded) always reloads.
      - At least one of O(state) and O(enabled) are required.
      - Note that V(reloaded) starts the service if it is not already started, even if your chosen init system would not normally.
  enabled:
    type: bool
    description:
      - Whether the service should start on boot.
      - At least one of O(state) and O(enabled) are required.
"""

EXAMPLES = r"""
- name: Example action to start service httpd, if not running
  community.general.simpleinit_msb:
    name: httpd
    state: started

- name: Example action to stop service httpd, if running
  community.general.simpleinit_msb:
    name: httpd
    state: stopped

- name: Example action to restart service httpd, in all cases
  community.general.simpleinit_msb:
    name: httpd
    state: restarted

- name: Example action to reload service httpd, in all cases
  community.general.simpleinit_msb:
    name: httpd
    state: reloaded

- name: Example action to enable service httpd, and not touch the running state
  community.general.simpleinit_msb:
    name: httpd
    enabled: true
"""

import os
import re

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.service import daemonize


class SimpleinitMSB:
    """
    Main simpleinit-msb service manipulation class
    """

    def __init__(self, module):
        self.module = module
        self.name = module.params["name"]
        self.state = module.params["state"]
        self.enable = module.params["enabled"]
        self.changed = False
        self.running = None
        self.action = None
        self.telinit_cmd = None
        self.svc_change = False

    def execute_command(self, cmd, daemon=False):
        if not daemon:
            return self.module.run_command(cmd)
        else:
            return daemonize(self.module, cmd)

    def check_service_changed(self):
        if self.state and self.running is None:
            self.module.fail_json(msg="failed determining service state, possible typo of service name?")
        # Find out if state has changed
        if not self.running and self.state in ["started", "running", "reloaded"]:
            self.svc_change = True
        elif self.running and self.state in ["stopped", "reloaded"]:
            self.svc_change = True
        elif self.state == "restarted":
            self.svc_change = True
        if self.module.check_mode and self.svc_change:
            self.module.exit_json(changed=True, msg="service state changed")

    def modify_service_state(self):
        # Only do something if state will change
        if self.svc_change:
            # Control service
            if self.state in ["started", "running"]:
                self.action = "start"
            elif not self.running and self.state == "reloaded":
                self.action = "start"
            elif self.state == "stopped":
                self.action = "stop"
            elif self.state == "reloaded":
                self.action = "reload"
            elif self.state == "restarted":
                self.action = "restart"

            if self.module.check_mode:
                self.module.exit_json(changed=True, msg="changing service state")

            return self.service_control()
        else:
            # If nothing needs to change just say all is well
            rc = 0
            err = ""
            out = ""
            return rc, out, err

    def get_service_tools(self):
        paths = ["/sbin", "/usr/sbin", "/bin", "/usr/bin"]
        binaries = ["telinit"]
        location = dict()

        for binary in binaries:
            location[binary] = self.module.get_bin_path(binary, opt_dirs=paths)

        if location.get("telinit", False) and os.path.exists("/etc/init.d/smgl_init"):
            self.telinit_cmd = location["telinit"]

        if self.telinit_cmd is None:
            self.module.fail_json(msg="cannot find telinit script for simpleinit-msb, aborting...")

    def get_service_status(self):
        self.action = "status"
        rc, status_stdout, status_stderr = self.service_control()

        if self.running is None and status_stdout.count("\n") <= 1:
            cleanout = status_stdout.lower().replace(self.name.lower(), "")

            if "is not running" in cleanout:
                self.running = False
            elif "is running" in cleanout:
                self.running = True

        return self.running

    def service_enable(self):
        # Check if the service is already enabled/disabled
        if not self.enable ^ self.service_enabled():
            return

        action = f"boot{'enable' if self.enable else 'disable'}"

        (rc, out, err) = self.execute_command(f"{self.telinit_cmd} {action} {self.name}")

        self.changed = True

        for line in err.splitlines():
            if self.enable and line.find("already enabled") != -1:
                self.changed = False
                break
            if not self.enable and line.find("already disabled") != -1:
                self.changed = False
                break

        if not self.changed:
            return

        return (rc, out, err)

    def service_enabled(self):
        self.service_exists()

        (rc, out, err) = self.execute_command(f"{self.telinit_cmd} {self.enable}d")

        service_enabled = not self.enable

        rex = re.compile(rf"^{self.name}$")

        for line in out.splitlines():
            if rex.match(line):
                service_enabled = True if self.enable else False
                break

        return service_enabled

    def service_exists(self):
        (rc, out, err) = self.execute_command(f"{self.telinit_cmd} list")

        service_exists = False

        rex = re.compile(rf"^\w+\s+{self.name}$")

        for line in out.splitlines():
            if rex.match(line):
                service_exists = True
                break

        if not service_exists:
            self.module.fail_json(msg=f"telinit could not find the requested service: {self.name}")

    def service_control(self):
        self.service_exists()

        svc_cmd = f"{self.telinit_cmd} run {self.name}"

        rc_state, stdout, stderr = self.execute_command(f"{svc_cmd} {self.action}", daemon=True)

        return (rc_state, stdout, stderr)


def build_module():
    return AnsibleModule(
        argument_spec=dict(
            name=dict(required=True, aliases=["service"]),
            state=dict(choices=["running", "started", "stopped", "restarted", "reloaded"]),
            enabled=dict(type="bool"),
        ),
        supports_check_mode=True,
        required_one_of=[["state", "enabled"]],
    )


def main():
    module = build_module()

    service = SimpleinitMSB(module)

    rc = 0
    out = ""
    err = ""
    result = {}
    result["name"] = service.name

    # Find service management tools
    service.get_service_tools()

    # Enable/disable service startup at boot if requested
    if service.module.params["enabled"] is not None:
        service.service_enable()
        result["enabled"] = service.enable

    if module.params["state"] is None:
        # Not changing the running state, so bail out now.
        result["changed"] = service.changed
        module.exit_json(**result)

    result["state"] = service.state

    service.get_service_status()

    # Calculate if request will change service state
    service.check_service_changed()

    # Modify service state if necessary
    (rc, out, err) = service.modify_service_state()

    if rc != 0:
        if err:
            module.fail_json(msg=err)
        else:
            module.fail_json(msg=out)

    result["changed"] = service.changed | service.svc_change
    if service.module.params["enabled"] is not None:
        result["enabled"] = service.module.params["enabled"]

    if not service.module.params["state"]:
        status = service.get_service_status()
        if status is None:
            result["state"] = "absent"
        elif status is False:
            result["state"] = "started"
        else:
            result["state"] = "stopped"
    else:
        # as we may have just bounced the service the service command may not
        # report accurate state at this moment so just show what we ran
        if service.module.params["state"] in ["started", "restarted", "running", "reloaded"]:
            result["state"] = "started"
        else:
            result["state"] = "stopped"

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019-2020, Andrew Klaus <andrewklaus@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: syspatch

short_description: Manage OpenBSD system patches


description:
  - Manage OpenBSD system patches using syspatch.
extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  revert:
    description:
      - Revert system patches.
    type: str
    choices: [all, one]

author:
  - Andrew Klaus (@precurse)
"""

EXAMPLES = r"""
- name: Apply all available system patches
  community.general.syspatch:

- name: Revert last patch
  community.general.syspatch:
    revert: one

- name: Revert all patches
  community.general.syspatch:
    revert: all

# NOTE: You can reboot automatically if a patch requires it:
- name: Apply all patches and store result
  community.general.syspatch:
  register: syspatch

- name: Reboot if patch requires it
  ansible.builtin.reboot:
  when: syspatch.reboot_needed
"""

RETURN = r"""
reboot_needed:
  description: Whether or not a reboot is required after an update.
  returned: always
  type: bool
  sample: true
"""

from ansible.module_utils.basic import AnsibleModule


def run_module():
    # define available arguments/parameters a user can pass to the module
    module_args = dict(revert=dict(type="str", choices=["all", "one"]))

    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    result = syspatch_run(module)

    module.exit_json(**result)


def syspatch_run(module):
    cmd = module.get_bin_path("syspatch", True)
    changed = False
    reboot_needed = False

    # Set safe defaults for run_flag and check_flag
    run_flag = ["-c"]
    check_flag = ["-c"]
    if module.params["revert"]:
        check_flag = ["-l"]

        if module.params["revert"] == "all":
            run_flag = ["-R"]
        else:
            run_flag = ["-r"]
    else:
        check_flag = ["-c"]
        run_flag = []

    # Run check command
    rc, out, err = module.run_command([cmd] + check_flag)

    if rc != 0:
        module.fail_json(msg=f"Command {cmd} failed rc={rc}, out={out}, err={err}")

    if len(out) > 0:
        # Changes pending
        change_pending = True
    else:
        # No changes pending
        change_pending = False

    if module.check_mode:
        changed = change_pending
    elif change_pending:
        rc, out, err = module.run_command([cmd] + run_flag)

        # Workaround syspatch ln bug:
        # http://openbsd-archive.7691.n7.nabble.com/Warning-applying-latest-syspatch-td354250.html
        if rc != 0 and err != "ln: /usr/X11R6/bin/X: No such file or directory\n":
            module.fail_json(msg=f"Command {cmd} failed rc={rc}, out={out}, err={err}")
        elif out.lower().find("create unique kernel") >= 0:
            # Kernel update applied
            reboot_needed = True
        elif out.lower().find("syspatch updated itself") >= 0:
            module.warn("Syspatch was updated. Please run syspatch again.")

        # If no stdout, then warn user
        if len(out) == 0:
            module.warn("syspatch had suggested changes, but stdout was empty.")

        changed = True
    else:
        changed = False

    return dict(
        changed=changed,
        reboot_needed=reboot_needed,
        rc=rc,
        stderr=err,
        stdout=out,
    )


def main():
    run_module()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017 Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: oneview_fcoe_network_info
short_description: Retrieve the information about one or more of the OneView FCoE Networks
description:
  - Retrieve the information about one or more of the FCoE Networks from OneView.
requirements:
  - hpOneView >= 2.0.1
author:
  - Felipe Bulsoni (@fgbulsoni)
  - Thiago Miotto (@tmiotto)
  - Adriane Cardozo (@adriane-cardozo)
attributes:
  check_mode:
    version_added: 3.3.0
    # This was backported to 2.5.4 and 1.3.11 as well, since this was a bugfix
options:
  name:
    description:
      - FCoE Network name.
    type: str
extends_documentation_fragment:
  - community.general._oneview
  - community.general._oneview.factsparams
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Gather information about all FCoE Networks
  community.general.oneview_fcoe_network_info:
    config: /etc/oneview/oneview_config.json
  delegate_to: localhost
  register: result

- name: Print fetched information about FCoE Networks
  ansible.builtin.debug:
    msg: "{{ result.fcoe_networks }}"

- name: Gather paginated, filtered and sorted information about FCoE Networks
  community.general.oneview_fcoe_network_info:
    config: /etc/oneview/oneview_config.json
    params:
      start: 0
      count: 3
      sort: 'name:descending'
      filter: 'vlanId=2'
  delegate_to: localhost
  register: result

- name: Print fetched information about paginated, filtered and sorted list of FCoE Networks
  ansible.builtin.debug:
    msg: "{{ result.fcoe_networks }}"

- name: Gather information about a FCoE Network by name
  community.general.oneview_fcoe_network_info:
    config: /etc/oneview/oneview_config.json
    name: Test FCoE Network Information
  delegate_to: localhost
  register: result

- name: Print fetched information about FCoE Network found by name
  ansible.builtin.debug:
    msg: "{{ result.fcoe_networks }}"
"""

RETURN = r"""
fcoe_networks:
  description: Has all the OneView information about the FCoE Networks.
  returned: Always, but can be null.
  type: dict
"""

from ansible_collections.community.general.plugins.module_utils._oneview import OneViewModuleBase


class FcoeNetworkInfoModule(OneViewModuleBase):
    def __init__(self):
        argument_spec = dict(
            name=dict(type="str"),
            params=dict(type="dict"),
        )

        super().__init__(
            additional_arg_spec=argument_spec,
            supports_check_mode=True,
        )

    def execute_module(self):
        if self.module.params["name"]:
            fcoe_networks = self.oneview_client.fcoe_networks.get_by("name", self.module.params["name"])
        else:
            fcoe_networks = self.oneview_client.fcoe_networks.get_all(**self.facts_params)

        return dict(changed=False, fcoe_networks=fcoe_networks)


def main():
    FcoeNetworkInfoModule().run()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: sl_vm
short_description: Create or cancel a virtual instance in SoftLayer
description:
  - Creates or cancels SoftLayer instances.
  - When created, optionally waits for it to be 'running'.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  instance_id:
    description:
      - Instance ID of the virtual instance to perform action option.
    type: str
  hostname:
    description:
      - Hostname to be provided to a virtual instance.
    type: str
  domain:
    description:
      - Domain name to be provided to a virtual instance.
    type: str
  datacenter:
    description:
      - Datacenter for the virtual instance to be deployed.
    type: str
    choices:
      - ams01
      - ams03
      - che01
      - dal01
      - dal05
      - dal06
      - dal09
      - dal10
      - dal12
      - dal13
      - fra02
      - fra04
      - fra05
      - hkg02
      - hou02
      - lon02
      - lon04
      - lon06
      - mel01
      - mex01
      - mil01
      - mon01
      - osl01
      - par01
      - sao01
      - sea01
      - seo01
      - sjc01
      - sjc03
      - sjc04
      - sng01
      - syd01
      - syd04
      - tok02
      - tor01
      - wdc01
      - wdc04
      - wdc06
      - wdc07
  tags:
    description:
      - Tag or list of tags to be provided to a virtual instance.
    type: str
  hourly:
    description:
      - Flag to determine if the instance should be hourly billed.
    type: bool
    default: true
  private:
    description:
      - Flag to determine if the instance should be private only.
    type: bool
    default: false
  dedicated:
    description:
      - Flag to determine if the instance should be deployed in dedicated space.
    type: bool
    default: false
  local_disk:
    description:
      - Flag to determine if local disk should be used for the new instance.
    type: bool
    default: true
  cpus:
    description:
      - Count of cpus to be assigned to new virtual instance.
    type: int
    choices: [1, 2, 4, 8, 16, 32, 56]
  memory:
    description:
      - Amount of memory to be assigned to new virtual instance.
    type: int
    choices: [1024, 2048, 4096, 6144, 8192, 12288, 16384, 32768, 49152, 65536, 131072, 247808]
  flavor:
    description:
      - Specify which SoftLayer flavor template to use instead of cpus and memory.
    version_added: '0.2.0'
    type: str
  disks:
    description:
      - List of disk sizes to be assigned to new virtual instance.
    default: [25]
    type: list
    elements: int
  os_code:
    description:
      - OS Code to be used for new virtual instance.
    type: str
  image_id:
    description:
      - Image Template to be used for new virtual instance.
    type: str
  nic_speed:
    description:
      - NIC Speed to be assigned to new virtual instance.
    choices: [10, 100, 1000]
    type: int
  public_vlan:
    description:
      - VLAN by its ID to be assigned to the public NIC.
    type: str
  private_vlan:
    description:
      - VLAN by its ID to be assigned to the private NIC.
    type: str
  ssh_keys:
    description:
      - List of ssh keys by their ID to be assigned to a virtual instance.
    type: list
    elements: str
    default: []
  post_uri:
    description:
      - URL of a post provisioning script to be loaded and executed on virtual instance.
    type: str
  state:
    description:
      - Create, or cancel a virtual instance.
      - Specify V(present) for create, V(absent) to cancel.
    choices: [absent, present]
    default: present
    type: str
  wait:
    description:
      - Flag used to wait for active status before returning.
    type: bool
    default: true
  wait_time:
    description:
      - Time in seconds before wait returns.
    default: 600
    type: int
requirements:
  - softlayer >= 4.1.1
notes:
  - The C(softlayer-python) library, at version 6.2.6 (from Jan 2025), only supports Python version 3.8, 3.9 and 3.10.
author:
  - Matt Colton (@mcltn)
seealso:
  - name: SoftLayer API Python Client
    description: The SoftLayer API Python Client is required for this module.
    link: https://github.com/SoftLayer/softlayer-python
"""

EXAMPLES = r"""
- name: Build instance
  hosts: localhost
  gather_facts: false
  tasks:
    - name: Build instance request
      community.general.sl_vm:
        hostname: instance-1
        domain: anydomain.com
        datacenter: dal09
        tags: ansible-module-test
        hourly: true
        private: false
        dedicated: false
        local_disk: true
        cpus: 1
        memory: 1024
        disks: [25]
        os_code: UBUNTU_LATEST
        wait: false

- name: Build additional instances
  hosts: localhost
  gather_facts: false
  tasks:
    - name: Build instances request
      community.general.sl_vm:
        hostname: "{{ item.hostname }}"
        domain: "{{ item.domain }}"
        datacenter: "{{ item.datacenter }}"
        tags: "{{ item.tags }}"
        hourly: "{{ item.hourly }}"
        private: "{{ item.private }}"
        dedicated: "{{ item.dedicated }}"
        local_disk: "{{ item.local_disk }}"
        cpus: "{{ item.cpus }}"
        memory: "{{ item.memory }}"
        disks: "{{ item.disks }}"
        os_code: "{{ item.os_code }}"
        ssh_keys: "{{ item.ssh_keys }}"
        wait: "{{ item.wait }}"
      with_items:
        - hostname: instance-2
          domain: anydomain.com
          datacenter: dal09
          tags:
            - ansible-module-test
            - ansible-module-test-replicas
          hourly: true
          private: false
          dedicated: false
          local_disk: true
          cpus: 1
          memory: 1024
          disks:
            - 25
            - 100
          os_code: UBUNTU_LATEST
          ssh_keys: []
          wait: true
        - hostname: instance-3
          domain: anydomain.com
          datacenter: dal09
          tags:
            - ansible-module-test
            - ansible-module-test-replicas
          hourly: true
          private: false
          dedicated: false
          local_disk: true
          cpus: 1
          memory: 1024
          disks:
            - 25
            - 100
          os_code: UBUNTU_LATEST
          ssh_keys: []
          wait: true

- name: Cancel instances
  hosts: localhost
  gather_facts: false
  tasks:
    - name: Cancel by tag
      community.general.sl_vm:
        state: absent
        tags: ansible-module-test
"""

# TODO: Disabled RETURN as it is breaking the build for docs. Needs to be fixed.
RETURN = """#"""

import json
import time

try:
    import SoftLayer
    from SoftLayer import VSManager

    HAS_SL = True
    vsManager = VSManager(SoftLayer.create_client_from_env())
except ImportError:
    HAS_SL = False

from ansible.module_utils.basic import AnsibleModule

# TODO: get this info from API
STATES = ["present", "absent"]
DATACENTERS = [
    "ams01",
    "ams03",
    "che01",
    "dal01",
    "dal05",
    "dal06",
    "dal09",
    "dal10",
    "dal12",
    "dal13",
    "fra02",
    "fra04",
    "fra05",
    "hkg02",
    "hou02",
    "lon02",
    "lon04",
    "lon06",
    "mel01",
    "mex01",
    "mil01",
    "mon01",
    "osl01",
    "par01",
    "sao01",
    "sea01",
    "seo01",
    "sjc01",
    "sjc03",
    "sjc04",
    "sng01",
    "syd01",
    "syd04",
    "tok02",
    "tor01",
    "wdc01",
    "wdc04",
    "wdc06",
    "wdc07",
]
CPU_SIZES = [1, 2, 4, 8, 16, 32, 56]
MEMORY_SIZES = [1024, 2048, 4096, 6144, 8192, 12288, 16384, 32768, 49152, 65536, 131072, 247808]
INITIALDISK_SIZES = [25, 100]
LOCALDISK_SIZES = [25, 100, 150, 200, 300]
SANDISK_SIZES = [10, 20, 25, 30, 40, 50, 75, 100, 125, 150, 175, 200, 250, 300, 350, 400, 500, 750, 1000, 1500, 2000]
NIC_SPEEDS = [10, 100, 1000]


def create_virtual_instance(module):
    instances = vsManager.list_instances(
        hostname=module.params.get("hostname"),
        domain=module.params.get("domain"),
        datacenter=module.params.get("datacenter"),
    )

    if instances:
        return False, None

    # Check if OS or Image Template is provided (Can't be both, defaults to OS)
    if module.params.get("os_code") is not None and module.params.get("os_code") != "":
        module.params["image_id"] = ""
    elif module.params.get("image_id") is not None and module.params.get("image_id") != "":
        module.params["os_code"] = ""
        module.params["disks"] = []  # Blank out disks since it will use the template
    else:
        return False, None

    tags = module.params.get("tags")
    if isinstance(tags, list):
        tags = ",".join(map(str, module.params.get("tags")))

    instance = vsManager.create_instance(
        hostname=module.params.get("hostname"),
        domain=module.params.get("domain"),
        cpus=module.params.get("cpus"),
        memory=module.params.get("memory"),
        flavor=module.params.get("flavor"),
        hourly=module.params.get("hourly"),
        datacenter=module.params.get("datacenter"),
        os_code=module.params.get("os_code"),
        image_id=module.params.get("image_id"),
        local_disk=module.params.get("local_disk"),
        disks=module.params.get("disks"),
        ssh_keys=module.params.get("ssh_keys"),
        nic_speed=module.params.get("nic_speed"),
        private=module.params.get("private"),
        public_vlan=module.params.get("public_vlan"),
        private_vlan=module.params.get("private_vlan"),
        dedicated=module.params.get("dedicated"),
        post_uri=module.params.get("post_uri"),
        tags=tags,
    )

    if instance is not None and instance["id"] > 0:
        return True, instance
    else:
        return False, None


def wait_for_instance(module, id):
    instance = None
    completed = False
    wait_timeout = time.time() + module.params.get("wait_time")
    while not completed and wait_timeout > time.time():
        try:
            completed = vsManager.wait_for_ready(id, 10, 2)
            if completed:
                instance = vsManager.get_instance(id)
        except Exception:
            completed = False

    return completed, instance


def cancel_instance(module):
    canceled = True
    if module.params.get("instance_id") is None and (
        module.params.get("tags") or module.params.get("hostname") or module.params.get("domain")
    ):
        tags = module.params.get("tags")
        if isinstance(tags, str):
            tags = [module.params.get("tags")]
        instances = vsManager.list_instances(
            tags=tags, hostname=module.params.get("hostname"), domain=module.params.get("domain")
        )
        for instance in instances:
            try:
                vsManager.cancel_instance(instance["id"])
            except Exception:
                canceled = False
    elif module.params.get("instance_id") and module.params.get("instance_id") != 0:
        try:
            vsManager.cancel_instance(instance["id"])
        except Exception:
            canceled = False
    else:
        return False, None

    return canceled, None


def main():
    module = AnsibleModule(
        argument_spec=dict(
            instance_id=dict(type="str"),
            hostname=dict(type="str"),
            domain=dict(type="str"),
            datacenter=dict(type="str", choices=DATACENTERS),
            tags=dict(type="str"),
            hourly=dict(type="bool", default=True),
            private=dict(type="bool", default=False),
            dedicated=dict(type="bool", default=False),
            local_disk=dict(type="bool", default=True),
            cpus=dict(type="int", choices=CPU_SIZES),
            memory=dict(type="int", choices=MEMORY_SIZES),
            flavor=dict(type="str"),
            disks=dict(type="list", elements="int", default=[25]),
            os_code=dict(type="str"),
            image_id=dict(type="str"),
            nic_speed=dict(type="int", choices=NIC_SPEEDS),
            public_vlan=dict(type="str"),
            private_vlan=dict(type="str"),
            ssh_keys=dict(type="list", elements="str", default=[], no_log=False),
            post_uri=dict(type="str"),
            state=dict(type="str", default="present", choices=STATES),
            wait=dict(type="bool", default=True),
            wait_time=dict(type="int", default=600),
        )
    )

    if not HAS_SL:
        module.fail_json(msg="softlayer python library required for this module")

    if module.params.get("state") == "absent":
        (changed, instance) = cancel_instance(module)

    elif module.params.get("state") == "present":
        (changed, instance) = create_virtual_instance(module)
        if module.params.get("wait") is True and instance:
            (changed, instance) = wait_for_instance(module, instance["id"])

    module.exit_json(changed=changed, instance=json.loads(json.dumps(instance, default=lambda o: o.__dict__)))


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2015, Matt Makai <matthew.makai@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: sendgrid
short_description: Sends an email with the SendGrid API
description:
  - Sends an email with a SendGrid account through their API, not through the SMTP service.
notes:
  - This module is non-idempotent because it sends an email through the external API. It is idempotent only in the case that
    the module fails.
  - Like the other notification modules, this one requires an external dependency to work. In this case, you need an active
    SendGrid account.
  - In order to use O(api_key), O(cc), O(bcc), O(attachments), O(from_name), O(html_body), and O(headers) you must C(pip install
    sendgrid).
requirements:
  - sendgrid Python library 1.6.22 or lower (Sendgrid API V2 supported)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  username:
    type: str
    description:
      - Username for logging into the SendGrid account.
      - It is only required if O(api_key) is not supplied.
  password:
    type: str
    description:
      - Password that corresponds to the username.
      - It is only required if O(api_key) is not supplied.
  from_address:
    type: str
    description:
      - The address in the "from" field for the email.
    required: true
  to_addresses:
    type: list
    elements: str
    description:
      - A list with one or more recipient email addresses.
    required: true
  subject:
    type: str
    description:
      - The desired subject for the email.
    required: true
  api_key:
    type: str
    description:
      - Sendgrid API key to use instead of username/password.
  cc:
    type: list
    elements: str
    description:
      - A list of email addresses to cc.
  bcc:
    type: list
    elements: str
    description:
      - A list of email addresses to bcc.
  attachments:
    type: list
    elements: path
    description:
      - A list of relative or explicit paths of files you want to attach (7MB limit as per SendGrid docs).
  from_name:
    type: str
    description:
      - The name you want to appear in the from field, for example V(John Doe).
  html_body:
    description:
      - Whether the body is HTML content that should be rendered.
    type: bool
    default: false
  headers:
    type: dict
    description:
      - A dict to pass on as headers.
  body:
    type: str
    description:
      - The e-mail body content.
    required: true
author: "Matt Makai (@makaimc)"
"""

EXAMPLES = r"""
- name: Send an email to a single recipient that the deployment was successful
  community.general.sendgrid:
    username: "{{ sendgrid_username }}"
    password: "{{ sendgrid_password }}"
    from_address: "ansible@mycompany.com"
    to_addresses:
      - "ops@mycompany.com"
    subject: "Deployment success."
    body: "The most recent Ansible deployment was successful."
  delegate_to: localhost

- name: Send an email to more than one recipient that the build failed
  community.general.sendgrid:
    username: "{{ sendgrid_username }}"
    password: "{{ sendgrid_password }}"
    from_address: "build@mycompany.com"
    to_addresses:
      - "ops@mycompany.com"
      - "devteam@mycompany.com"
    subject: "Build failure!."
    body: "Unable to pull source repository from Git server."
  delegate_to: localhost
"""

# =======================================
# sendgrid module support methods
#
import os
import traceback
from urllib.parse import urlencode

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion

SENDGRID_IMP_ERR = None
try:
    import sendgrid

    HAS_SENDGRID = True
except ImportError:
    SENDGRID_IMP_ERR = traceback.format_exc()
    HAS_SENDGRID = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes
from ansible.module_utils.urls import fetch_url


def post_sendgrid_api(
    module,
    username,
    password,
    from_address,
    to_addresses,
    subject,
    body,
    api_key=None,
    cc=None,
    bcc=None,
    attachments=None,
    html_body=False,
    from_name=None,
    headers=None,
):
    if not HAS_SENDGRID:
        SENDGRID_URI = "https://api.sendgrid.com/api/mail.send.json"
        AGENT = "Ansible"
        data = {"api_user": username, "api_key": password, "from": from_address, "subject": subject, "text": body}
        encoded_data = urlencode(data)
        to_addresses_api = ""
        for recipient in to_addresses:
            recipient = to_bytes(recipient, errors="surrogate_or_strict")
            to_addresses_api += f"&to[]={recipient}"
        encoded_data += to_addresses_api

        headers = {
            "User-Agent": AGENT,
            "Content-type": "application/x-www-form-urlencoded",
            "Accept": "application/json",
        }
        return fetch_url(module, SENDGRID_URI, data=encoded_data, headers=headers, method="POST")
    else:
        # Remove this check when adding Sendgrid API v3 support
        if LooseVersion(sendgrid.version.__version__) > LooseVersion("1.6.22"):
            module.fail_json(msg="Please install sendgrid==1.6.22 or lower since module uses Sendgrid V2 APIs.")

        if api_key:
            sg = sendgrid.SendGridClient(api_key)
        else:
            sg = sendgrid.SendGridClient(username, password)

        message = sendgrid.Mail()
        message.set_subject(subject)

        for recip in to_addresses:
            message.add_to(recip)

        if cc:
            for recip in cc:
                message.add_cc(recip)
        if bcc:
            for recip in bcc:
                message.add_bcc(recip)

        if headers:
            message.set_headers(headers)

        if attachments:
            for f in attachments:
                name = os.path.basename(f)
                message.add_attachment(name, f)

        if from_name:
            message.set_from(f"{from_name} <{from_address}.")
        else:
            message.set_from(from_address)

        if html_body:
            message.set_html(body)
        else:
            message.set_text(body)

        return sg.send(message)


# =======================================
# Main
#


def main():
    module = AnsibleModule(
        argument_spec=dict(
            username=dict(),
            password=dict(no_log=True),
            api_key=dict(no_log=True),
            bcc=dict(type="list", elements="str"),
            cc=dict(type="list", elements="str"),
            headers=dict(type="dict"),
            from_address=dict(required=True),
            from_name=dict(),
            to_addresses=dict(required=True, type="list", elements="str"),
            subject=dict(required=True),
            body=dict(required=True),
            html_body=dict(default=False, type="bool"),
            attachments=dict(type="list", elements="path"),
        ),
        supports_check_mode=True,
        mutually_exclusive=[["api_key", "password"], ["api_key", "username"]],
        required_together=[["username", "password"]],
    )

    username = module.params["username"]
    password = module.params["password"]
    api_key = module.params["api_key"]
    bcc = module.params["bcc"]
    cc = module.params["cc"]
    headers = module.params["headers"]
    from_name = module.params["from_name"]
    from_address = module.params["from_address"]
    to_addresses = module.params["to_addresses"]
    subject = module.params["subject"]
    body = module.params["body"]
    html_body = module.params["html_body"]
    attachments = module.params["attachments"]

    sendgrid_lib_args = [api_key, bcc, cc, headers, from_name, html_body, attachments]

    if any(lib_arg is not None for lib_arg in sendgrid_lib_args) and not HAS_SENDGRID:
        reason = (
            "when using any of the following arguments: api_key, bcc, cc, headers, from_name, html_body, attachments"
        )
        module.fail_json(msg=missing_required_lib("sendgrid", reason=reason), exception=SENDGRID_IMP_ERR)

    response, info = post_sendgrid_api(
        module,
        username,
        password,
        from_address,
        to_addresses,
        subject,
        body,
        attachments=attachments,
        bcc=bcc,
        cc=cc,
        headers=headers,
        html_body=html_body,
        api_key=api_key,
    )

    if not HAS_SENDGRID:
        if info["status"] != 200:
            module.fail_json(msg=f"unable to send email through SendGrid API: {info['msg']}")
    else:
        if response != 200:
            module.fail_json(msg=f"unable to send email through SendGrid API: {info['message']}")

    module.exit_json(msg=subject, changed=False)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2013, Alexander Winkler <mail () winkler-alexander.de>
# based on svr4pkg by
#  Boyd Adamson <boyd () boydadamson.com> (2012)
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pkgutil
short_description: OpenCSW package management on Solaris
description:
  - This module installs, updates and removes packages from the OpenCSW project for Solaris.
  - Unlike the M(community.general.svr4pkg) module, it resolves and downloads dependencies.
  - See U(https://www.opencsw.org/) for more information about the project.
author:
  - Alexander Winkler (@dermute)
  - David Ponessa (@scathatheworm)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
    details:
      - In order to check the availability of packages, the catalog cache under C(/var/opt/csw/pkgutil) may be refreshed even
        in check mode.
  diff_mode:
    support: none
options:
  name:
    description:
      - The name of the package.
      - When using O(state=latest), this can be V('*'), which updates all installed packages managed by pkgutil.
    type: list
    required: true
    elements: str
    aliases: [pkg]
  site:
    description:
      - The repository path to install the package from.
      - Its global definition is in C(/etc/opt/csw/pkgutil.conf).
    type: str
  state:
    description:
      - Whether to install (V(present)/V(installed)), or remove (V(absent)/V(removed)) packages.
      - The upgrade (V(latest)) operation updates/installs the packages to the latest version available.
    type: str
    required: true
    choices: [absent, installed, latest, present, removed]
  update_catalog:
    description:
      - If you always want to refresh your catalog from the mirror, even when it is not stale, set this to V(true).
    type: bool
    default: false
  force:
    description:
      - To allow the update process to downgrade packages to match what is present in the repository, set this to V(true).
      - This is useful for rolling back to stable from testing, or similar operations.
    type: bool
    default: false
    version_added: 1.2.0
"""

EXAMPLES = r"""
- name: Install a package
  community.general.pkgutil:
    name: CSWcommon
    state: present

- name: Install a package from a specific repository
  community.general.pkgutil:
    name: CSWnrpe
    site: ftp://myinternal.repo/opencsw/kiel
    state: latest

- name: Remove a package
  community.general.pkgutil:
    name: CSWtop
    state: absent

- name: Install several packages
  community.general.pkgutil:
    name:
      - CSWsudo
      - CSWtop
    state: present

- name: Update all packages
  community.general.pkgutil:
    name: '*'
    state: latest

- name: Update all packages and force versions to match latest in catalog
  community.general.pkgutil:
    name: '*'
    state: latest
    force: true
"""

RETURN = r""" # """

from ansible.module_utils.basic import AnsibleModule


def packages_not_installed(module, names):
    """Check if each package is installed and return list of the ones absent"""
    pkgs = []
    for pkg in names:
        rc, out, err = run_command(module, ["pkginfo", "-q", pkg])
        if rc != 0:
            pkgs.append(pkg)
    return pkgs


def packages_installed(module, names):
    """Check if each package is installed and return list of the ones present"""
    pkgs = []
    for pkg in names:
        if not pkg.startswith("CSW"):
            continue
        rc, out, err = run_command(module, ["pkginfo", "-q", pkg])
        if rc == 0:
            pkgs.append(pkg)
    return pkgs


def packages_not_latest(module, names, site, update_catalog):
    """Check status of each package and return list of the ones with an upgrade available"""
    cmd = ["pkgutil"]
    if update_catalog:
        cmd.append("-U")
    cmd.append("-c")
    if site is not None:
        cmd.extend(["-t", site])
    if names != ["*"]:
        cmd.extend(names)
    rc, out, err = run_command(module, cmd)

    # Find packages in the catalog which are not up to date
    packages = []
    for line in out.split("\n")[1:-1]:
        if "catalog" not in line and "SAME" not in line:
            packages.append(line.split(" ")[0])

    # Remove duplicates
    return list(set(packages))


def run_command(module, cmd, **kwargs):
    progname = cmd[0]
    cmd[0] = module.get_bin_path(progname, True, ["/opt/csw/bin"])
    return module.run_command(cmd, **kwargs)


def package_install(module, state, pkgs, site, update_catalog, force):
    cmd = ["pkgutil"]
    if module.check_mode:
        cmd.append("-n")
    cmd.append("-iy")
    if update_catalog:
        cmd.append("-U")
    if site is not None:
        cmd.extend(["-t", site])
    if force:
        cmd.append("-f")
    cmd.extend(pkgs)
    return run_command(module, cmd)


def package_upgrade(module, pkgs, site, update_catalog, force):
    cmd = ["pkgutil"]
    if module.check_mode:
        cmd.append("-n")
    cmd.append("-uy")
    if update_catalog:
        cmd.append("-U")
    if site is not None:
        cmd.extend(["-t", site])
    if force:
        cmd.append("-f")
    cmd += pkgs
    return run_command(module, cmd)


def package_uninstall(module, pkgs):
    cmd = ["pkgutil"]
    if module.check_mode:
        cmd.append("-n")
    cmd.append("-ry")
    cmd.extend(pkgs)
    return run_command(module, cmd)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="list", elements="str", required=True, aliases=["pkg"]),
            state=dict(type="str", required=True, choices=["absent", "installed", "latest", "present", "removed"]),
            site=dict(type="str"),
            update_catalog=dict(type="bool", default=False),
            force=dict(type="bool", default=False),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}
    name = module.params["name"]
    state = module.params["state"]
    site = module.params["site"]
    update_catalog = module.params["update_catalog"]
    force = module.params["force"]

    rc = None
    out = ""
    err = ""
    result = dict(
        name=name,
        state=state,
    )

    if state in ["installed", "present"]:
        # Fail with an explicit error when trying to "install" '*'
        if name == ["*"]:
            module.fail_json(msg="Can not use 'state: present' with name: '*'")

        # Build list of packages that are actually not installed from the ones requested
        pkgs = packages_not_installed(module, name)

        # If the package list is empty then all packages are already present
        if pkgs == []:
            module.exit_json(changed=False)

        (rc, out, err) = package_install(module, state, pkgs, site, update_catalog, force)
        if rc != 0:
            module.fail_json(msg=(err or out))

    elif state in ["latest"]:
        # When using latest for *
        if name == ["*"]:
            # Check for packages that are actually outdated
            pkgs = packages_not_latest(module, name, site, update_catalog)

            # If the package list comes up empty, everything is already up to date
            if pkgs == []:
                module.exit_json(changed=False)

            # If there are packages to update, just empty the list and run the command without it
            # pkgutil logic is to update all when run without packages names
            pkgs = []
            (rc, out, err) = package_upgrade(module, pkgs, site, update_catalog, force)
            if rc != 0:
                module.fail_json(msg=(err or out))
        else:
            # Build list of packages that are either outdated or not installed
            pkgs = packages_not_installed(module, name)
            pkgs += packages_not_latest(module, name, site, update_catalog)

            # If the package list is empty that means all packages are installed and up to date
            if pkgs == []:
                module.exit_json(changed=False)

            (rc, out, err) = package_upgrade(module, pkgs, site, update_catalog, force)
            if rc != 0:
                module.fail_json(msg=(err or out))

    elif state in ["absent", "removed"]:
        # Build list of packages requested for removal that are actually present
        pkgs = packages_installed(module, name)

        # If the list is empty, no packages need to be removed
        if pkgs == []:
            module.exit_json(changed=False)

        (rc, out, err) = package_uninstall(module, pkgs)
        if rc != 0:
            module.fail_json(msg=(err or out))

    if rc is None:
        # pkgutil was not executed because the package was already present/absent/up to date
        result["changed"] = False
    elif rc == 0:
        result["changed"] = True
    else:
        result["changed"] = False
        result["failed"] = True

    if out:
        result["stdout"] = out
    if err:
        result["stderr"] = err

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Scaleway Container registry management module
#
# Copyright (c) 2022, Guillaume MARTINEZ <lunik@tiwabbit.fr>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: scaleway_container_registry
short_description: Scaleway Container registry management module
version_added: 5.8.0
author: Guillaume MARTINEZ (@Lunik)
description:
  - This module manages container registries on Scaleway account.
extends_documentation_fragment:
  - community.general._scaleway
  - community.general._scaleway_waitable_resource
  - community.general._attributes
  - community.general._scaleway.actiongroup_scaleway

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  action_group:
    version_added: 11.3.0

options:
  state:
    type: str
    description:
      - Indicate desired state of the container registry.
    default: present
    choices:
      - present
      - absent

  project_id:
    type: str
    description:
      - Project identifier.
    required: true

  region:
    type: str
    description:
      - Scaleway region to use (for example V(fr-par)).
    required: true
    choices:
      - fr-par
      - nl-ams
      - pl-waw

  name:
    type: str
    description:
      - Name of the container registry.
    required: true

  description:
    description:
      - Description of the container registry.
    type: str
    default: ''

  privacy_policy:
    type: str
    description:
      - Default visibility policy.
      - Everyone can pull images from a V(public) registry.
    choices:
      - public
      - private
    default: private
"""

EXAMPLES = r"""
- name: Create a container registry
  community.general.scaleway_container_registry:
    project_id: '{{ scw_project }}'
    state: present
    region: fr-par
    name: my-awesome-container-registry
  register: container_registry_creation_task

- name: Make sure container registry is deleted
  community.general.scaleway_container_registry:
    project_id: '{{ scw_project }}'
    state: absent
    region: fr-par
    name: my-awesome-container-registry
"""

RETURN = r"""
container_registry:
  description: The container registry information.
  returned: when O(state=present)
  type: dict
  sample:
    created_at: "2022-10-14T09:51:07.949716Z"
    description: Managed by Ansible
    endpoint: rg.fr-par.scw.cloud/my-awesome-registry
    id: 0d7d5270-7864-49c2-920b-9fd6731f3589
    image_count: 0
    is_public: false
    name: my-awesome-registry
    organization_id: 10697b59-5c34-4d24-8d15-9ff2d3b89f58
    project_id: 3da4f0b2-06be-4773-8ec4-5dfa435381be
    region: fr-par
    size: 0
    status: ready
    status_message: ""
    updated_at: "2022-10-14T09:51:07.949716Z"
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._scaleway import (
    SCALEWAY_REGIONS,
    Scaleway,
    resource_attributes_should_be_changed,
    scaleway_argument_spec,
    scaleway_waitable_resource_argument_spec,
)

STABLE_STATES = ("ready", "absent")

MUTABLE_ATTRIBUTES = ("description", "is_public")


def payload_from_wished_cr(wished_cr):
    payload = {
        "project_id": wished_cr["project_id"],
        "name": wished_cr["name"],
        "description": wished_cr["description"],
        "is_public": wished_cr["privacy_policy"] == "public",
    }

    return payload


def absent_strategy(api, wished_cr):
    changed = False

    cr_list = api.fetch_all_resources("namespaces")
    cr_lookup = {cr["name"]: cr for cr in cr_list}

    if wished_cr["name"] not in cr_lookup:
        return changed, {}

    target_cr = cr_lookup[wished_cr["name"]]
    changed = True
    if api.module.check_mode:
        return changed, {"status": "Container registry would be destroyed"}

    api.wait_to_complete_state_transition(resource=target_cr, stable_states=STABLE_STATES, force_wait=True)
    response = api.delete(path=f"{api.api_path}/{target_cr['id']}")
    if not response.ok:
        api.module.fail_json(msg=f"Error deleting container registry [{response.status_code}: {response.json}]")

    api.wait_to_complete_state_transition(resource=target_cr, stable_states=STABLE_STATES)
    return changed, response.json


def present_strategy(api, wished_cr):
    changed = False

    cr_list = api.fetch_all_resources("namespaces")
    cr_lookup = {cr["name"]: cr for cr in cr_list}

    payload_cr = payload_from_wished_cr(wished_cr)

    if wished_cr["name"] not in cr_lookup:
        changed = True
        if api.module.check_mode:
            return changed, {"status": "A container registry would be created."}

        # Create container registry
        api.warn(payload_cr)
        creation_response = api.post(path=api.api_path, data=payload_cr)

        if not creation_response.ok:
            msg = f"Error during container registry creation: {creation_response.info['msg']}: '{creation_response.json['message']}' ({creation_response.json})"
            api.module.fail_json(msg=msg)

        api.wait_to_complete_state_transition(resource=creation_response.json, stable_states=STABLE_STATES)
        response = api.get(path=f"{api.api_path}/{creation_response.json['id']}")
        return changed, response.json

    target_cr = cr_lookup[wished_cr["name"]]
    patch_payload = resource_attributes_should_be_changed(
        target=target_cr,
        wished=payload_cr,
        verifiable_mutable_attributes=MUTABLE_ATTRIBUTES,
        mutable_attributes=MUTABLE_ATTRIBUTES,
    )

    if not patch_payload:
        return changed, target_cr

    changed = True
    if api.module.check_mode:
        return changed, {"status": "Container registry attributes would be changed."}

    cr_patch_response = api.patch(path=f"{api.api_path}/{target_cr['id']}", data=patch_payload)

    if not cr_patch_response.ok:
        api.module.fail_json(
            msg=f"Error during container registry attributes update: [{cr_patch_response.status_code}: {cr_patch_response.json['message']}]"
        )

    api.wait_to_complete_state_transition(resource=target_cr, stable_states=STABLE_STATES)
    response = api.get(path=f"{api.api_path}/{target_cr['id']}")
    return changed, response.json


state_strategy = {"present": present_strategy, "absent": absent_strategy}


def core(module):
    region = module.params["region"]
    wished_container_registry = {
        "state": module.params["state"],
        "project_id": module.params["project_id"],
        "name": module.params["name"],
        "description": module.params["description"],
        "privacy_policy": module.params["privacy_policy"],
    }

    api = Scaleway(module=module)
    api.api_path = f"registry/v1/regions/{region}/namespaces"

    changed, summary = state_strategy[wished_container_registry["state"]](api=api, wished_cr=wished_container_registry)

    module.exit_json(changed=changed, container_registry=summary)


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(scaleway_waitable_resource_argument_spec())
    argument_spec.update(
        dict(
            state=dict(type="str", default="present", choices=["absent", "present"]),
            project_id=dict(type="str", required=True),
            region=dict(type="str", required=True, choices=SCALEWAY_REGIONS),
            name=dict(type="str", required=True),
            description=dict(type="str", default=""),
            privacy_policy=dict(type="str", default="private", choices=["public", "private"]),
        )
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    core(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_appliance_firewall_multicast_forwarding_by_network_info
short_description: Information module for organizations _appliance _firewall _multicast
  _forwarding _by _network
description:
  - Information module for Organizations Appliance Firewall Multicast Forwarding By Network Info.
  - Get all organizations _appliance _firewall _multicast _forwarding _by _network.
  - List Static Multicasting forwarding settings for MX networks.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - Information module for Organizations Appliance Firewall Multicast Forwarding By Network Info.
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - Information module for Organizations Appliance Firewall Multicast Forwarding By Network Info.
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - Information module for Organizations Appliance Firewall Multicast Forwarding By Network Info.
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - Information module for Organizations Appliance Firewall Multicast Forwarding By Network Info.
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - Information module for Organizations Appliance Firewall Multicast Forwarding By Network Info.
      - NetworkIds query parameter. Optional parameter to filter the results by network
        IDs.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance getOrganizationApplianceFirewallMulticastForwardingByNetwork
    description: Complete reference of the getOrganizationApplianceFirewallMulticastForwardingByNetwork
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-appliance-firewall-multicast-forwarding-by-network
notes:
  - SDK Method used are
    appliance.Appliance.get_organization_appliance_firewall_multicast_forwarding_by_network,
  - Paths used are
    get /organizations/{organizationId}/appliance/firewall/multicastForwarding/byNetwork,
"""

EXAMPLES = r"""
- name: Get all organizations _appliance _firewall _multicast _forwarding _by _network
  cisco.meraki.organizations_appliance_firewall_multicast_forwarding_by_network_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "network": {
            "id": "string",
            "name": "string"
          },
          "rules": [
            {
              "description": "string",
              "address": "string",
              "vlanIds": [
                "string"
              ]
            }
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python

# Copyright (c) 2016, Deepak Kothandan <deepak.kothandan@outlook.com>
# Copyright (c) 2015, Stefan Berggren <nsg@nsg.cc>
# Copyright (c) 2014, Ramon de la Fuente <ramon@delafuente.nl>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: rocketchat
short_description: Send notifications to Rocket Chat
description:
  - This module sends notifications to Rocket Chat through the Incoming WebHook integration.
author: "Ramon de la Fuente (@ramondelafuente)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  domain:
    type: str
    description:
      - The domain for your environment without protocol. (For example V(example.com) or V(chat.example.com)).
    required: true
  token:
    type: str
    description:
      - Rocket Chat Incoming Webhook integration token. This provides authentication to Rocket Chat's Incoming webhook for
        posting messages.
    required: true
  protocol:
    type: str
    description:
      - Specify the protocol used to send notification messages before the webhook URL (that is, V(http) or V(https)).
    default: https
    choices:
      - 'http'
      - 'https'
  msg:
    type: str
    description:
      - Message to be sent.
  channel:
    type: str
    description:
      - Channel to send the message to. If absent, the message goes to the channel selected for the O(token) specified during
        the creation of webhook.
  username:
    type: str
    description:
      - This is the sender of the message.
    default: "Ansible"
  icon_url:
    type: str
    description:
      - URL for the message sender's icon.
    default: "https://docs.ansible.com/favicon/favicon.ico"
  icon_emoji:
    type: str
    description:
      - Emoji for the message sender. The representation for the available emojis can be got from Rocket Chat.
      - For example V(:thumbsup:).
      - If O(icon_emoji) is set, O(icon_url) is not used.
  link_names:
    type: int
    description:
      - Automatically create links for channels and usernames in O(msg).
    default: 1
    choices:
      - 1
      - 0
  validate_certs:
    description:
      - If V(false), SSL certificates are not validated. This should only be used on personally controlled sites using self-signed
        certificates.
    type: bool
    default: true
  color:
    type: str
    description:
      - Allow text to use default colors - use the default of V(normal) to not send a custom color bar at the start of the
        message.
    default: 'normal'
    choices:
      - 'normal'
      - 'good'
      - 'warning'
      - 'danger'
  attachments:
    type: list
    elements: dict
    description:
      - Define a list of attachments.
  is_pre740:
    description:
      - If V(true), the payload matches Rocket.Chat prior to 7.4.0 format. This format has been used by the module since its
        inception, but is no longer supported by Rocket.Chat 7.4.0.
      - The default value changed from V(true) to V(false) in community.general 13.0.0.
      - This parameter is going to be removed in a future release when Rocket.Chat 7.4.0 becomes the minimum supported version.
    type: bool
    default: false
    version_added: 10.5.0
"""

EXAMPLES = r"""
- name: Send notification message through Rocket Chat
  community.general.rocketchat:
    token: thetoken/generatedby/rocketchat
    domain: chat.example.com
    msg: '{{ inventory_hostname }} completed'
  delegate_to: localhost

- name: Send notification message through Rocket Chat all options
  community.general.rocketchat:
    domain: chat.example.com
    token: thetoken/generatedby/rocketchat
    msg: '{{ inventory_hostname }} completed'
    channel: "#ansible"
    username: 'Ansible on {{ inventory_hostname }}'
    icon_url: http://www.example.com/some-image-file.png
    link_names: 0
  delegate_to: localhost

- name: Insert a color bar in front of the message for visibility purposes and use the default webhook icon and name configured
    in rocketchat
  community.general.rocketchat:
    token: thetoken/generatedby/rocketchat
    domain: chat.example.com
    msg: '{{ inventory_hostname }} is alive!'
    color: good
    username: ''
    icon_url: ''
  delegate_to: localhost

- name: Use the attachments API
  community.general.rocketchat:
    token: thetoken/generatedby/rocketchat
    domain: chat.example.com
    attachments:
      - text: Display my system load on host A and B
        color: "#ff00dd"
        title: System load
        fields:
          - title: System A
            value: 'load average: 0,74, 0,66, 0,63'
            short: true
          - title: System B
            value: 'load average: 5,16, 4,64, 2,43'
            short: true
  delegate_to: localhost
"""


from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def build_payload_for_rocketchat(
    module, text, channel, username, icon_url, icon_emoji, link_names, color, attachments, is_pre740
):
    payload = {}
    if color == "normal" and text is not None:
        payload = dict(text=text)
    elif text is not None:
        payload = dict(attachments=[dict(text=text, color=color)])
    if channel is not None:
        if channel[0] == "#" or channel[0] == "@":
            payload["channel"] = channel
        else:
            payload["channel"] = f"#{channel}"
    if username is not None:
        payload["username"] = username
    if icon_emoji is not None:
        payload["icon_emoji"] = icon_emoji
    else:
        payload["icon_url"] = icon_url
    if link_names is not None:
        payload["link_names"] = link_names

    if attachments is not None:
        if "attachments" not in payload:
            payload["attachments"] = []

    if attachments is not None:
        for attachment in attachments:
            if "fallback" not in attachment:
                attachment["fallback"] = attachment["text"]
            payload["attachments"].append(attachment)

    payload = module.jsonify(payload)
    if is_pre740:
        payload = f"payload={payload}"
    return payload


def do_notify_rocketchat(module, domain, token, protocol, payload, is_pre740):
    if token.count("/") < 1:
        module.fail_json(msg="Invalid Token specified, provide a valid token")

    rocketchat_incoming_webhook = f"{protocol}://{domain}/hooks/{token}"

    headers = None
    if not is_pre740:
        headers = {"Content-type": "application/json"}
    response, info = fetch_url(module, rocketchat_incoming_webhook, data=payload, headers=headers)
    if info["status"] != 200:
        module.fail_json(msg=f"failed to send message, return status={info['status']}")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            domain=dict(type="str", required=True),
            token=dict(type="str", required=True, no_log=True),
            protocol=dict(type="str", default="https", choices=["http", "https"]),
            msg=dict(type="str"),
            channel=dict(type="str"),
            username=dict(type="str", default="Ansible"),
            icon_url=dict(type="str", default="https://docs.ansible.com/favicon/favicon.ico"),
            icon_emoji=dict(type="str"),
            link_names=dict(type="int", default=1, choices=[0, 1]),
            validate_certs=dict(default=True, type="bool"),
            color=dict(type="str", default="normal", choices=["normal", "good", "warning", "danger"]),
            attachments=dict(type="list", elements="dict"),
            is_pre740=dict(type="bool", default=False),
        )
    )

    domain = module.params["domain"]
    token = module.params["token"]
    protocol = module.params["protocol"]
    text = module.params["msg"]
    channel = module.params["channel"]
    username = module.params["username"]
    icon_url = module.params["icon_url"]
    icon_emoji = module.params["icon_emoji"]
    link_names = module.params["link_names"]
    color = module.params["color"]
    attachments = module.params["attachments"]
    is_pre740 = module.params["is_pre740"]

    payload = build_payload_for_rocketchat(
        module, text, channel, username, icon_url, icon_emoji, link_names, color, attachments, is_pre740
    )
    do_notify_rocketchat(module, domain, token, protocol, payload, is_pre740)

    module.exit_json(msg="OK")


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2025, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pacemaker_info
short_description: Gather information about Pacemaker cluster
author:
  - Dexter Le (@munchtoast)
version_added: 11.2.0
description:
  - Gather information about a Pacemaker cluster.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Gather Pacemaker cluster info
  community.general.pacemaker_info:
  register: result

- name: Debug cluster info
  ansible.builtin.debug:
    msg: "{{ result }}"
"""

RETURN = r"""
version:
  description: Pacemaker CLI version
  returned: always
  type: str
cluster_info:
  description: Cluster information such as the name, UUID, and nodes.
  returned: always
  type: dict
resource_info:
  description: All resources available on the cluster and their status.
  returned: success
  type: dict
stonith_info:
  description: All STONITH information on the cluster.
  returned: success
  type: dict
constraint_info:
  description: All cluster resource constraints on the cluster.
  returned: success
  type: dict
property_info:
  description: All properties present on the cluster.
  returned: success
  type: dict
"""

import json

from ansible_collections.community.general.plugins.module_utils._module_helper import ModuleHelper
from ansible_collections.community.general.plugins.module_utils._pacemaker import pacemaker_runner


class PacemakerInfo(ModuleHelper):
    module = dict(
        argument_spec=dict(),
        supports_check_mode=True,
    )
    info_vars = {
        "cluster_info": "cluster",
        "resource_info": "resource",
        "stonith_info": "stonith",
        "constraint_info": "constraint",
        "property_info": "property",
    }
    output_params = list(info_vars.keys())

    def __init_module__(self):
        self.runner = pacemaker_runner(self.module)
        with self.runner("version") as ctx:
            rc, out, err = ctx.run()
            self.vars.version = out.strip()

    def _process_command_output(self, cli_action=""):
        def process(rc, out, err):
            if rc != 0:
                self.do_raise(f"pcs {cli_action} config failed with error (rc={rc}): {err}")
            out = json.loads(out)
            return None if out == "" else out

        return process

    def _get_info(self, cli_action):
        with self.runner(
            "cli_action config output_format", output_process=self._process_command_output(cli_action)
        ) as ctx:
            return ctx.run(cli_action=cli_action, output_format="json")

    def __run__(self):
        for key, cli_action in sorted(self.info_vars.items()):
            self.vars.set(key, self._get_info(cli_action))


def main():
    PacemakerInfo.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017 Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: oneview_network_set
short_description: Manage HPE OneView Network Set resources
description:
  - Provides an interface to manage Network Set resources. Can create, update, or delete.
requirements:
  - hpOneView >= 4.0.0
author:
  - Felipe Bulsoni (@fgbulsoni)
  - Thiago Miotto (@tmiotto)
  - Adriane Cardozo (@adriane-cardozo)
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    description:
      - Indicates the desired state for the Network Set resource.
      - V(present) ensures data properties are compliant with OneView.
      - V(absent) removes the resource from OneView, if it exists.
    type: str
    default: present
    choices: ['present', 'absent']
  data:
    description:
      - List with the Network Set properties.
    type: dict
    required: true

extends_documentation_fragment:
  - community.general._oneview
  - community.general._oneview.validateetag
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Create a Network Set
  community.general.oneview_network_set:
    config: /etc/oneview/oneview_config.json
    state: present
    data:
      name: OneViewSDK Test Network Set
      networkUris:
        - Test Ethernet Network_1                                      # can be a name
        - /rest/ethernet-networks/e4360c9d-051d-4931-b2aa-7de846450dd8 # or a URI
  delegate_to: localhost

- name: Update the Network Set name to 'OneViewSDK Test Network Set - Renamed' and change the associated networks
  community.general.oneview_network_set:
    config: /etc/oneview/oneview_config.json
    state: present
    data:
      name: OneViewSDK Test Network Set
      newName: OneViewSDK Test Network Set - Renamed
      networkUris:
        - Test Ethernet Network_1
  delegate_to: localhost

- name: Delete the Network Set
  community.general.oneview_network_set:
    config: /etc/oneview/oneview_config.json
    state: absent
    data:
      name: OneViewSDK Test Network Set - Renamed
  delegate_to: localhost

- name: Update the Network set with two scopes
  community.general.oneview_network_set:
    config: /etc/oneview/oneview_config.json
    state: present
    data:
      name: OneViewSDK Test Network Set
      scopeUris:
        - /rest/scopes/01SC123456
        - /rest/scopes/02SC123456
  delegate_to: localhost
"""

RETURN = r"""
network_set:
  description: Has the facts about the Network Set.
  returned: On O(state=present), but can be null.
  type: dict
"""

from ansible_collections.community.general.plugins.module_utils._oneview import (
    OneViewModuleBase,
    OneViewModuleResourceNotFound,
)


class NetworkSetModule(OneViewModuleBase):
    MSG_CREATED = "Network Set created successfully."
    MSG_UPDATED = "Network Set updated successfully."
    MSG_DELETED = "Network Set deleted successfully."
    MSG_ALREADY_PRESENT = "Network Set is already present."
    MSG_ALREADY_ABSENT = "Network Set is already absent."
    MSG_ETHERNET_NETWORK_NOT_FOUND = "Ethernet Network not found: "
    RESOURCE_FACT_NAME = "network_set"

    argument_spec = dict(
        state=dict(default="present", choices=["present", "absent"]), data=dict(required=True, type="dict")
    )

    def __init__(self):
        super().__init__(additional_arg_spec=self.argument_spec, validate_etag_support=True)
        self.resource_client = self.oneview_client.network_sets

    def execute_module(self):
        resource = self.get_by_name(self.data.get("name"))

        if self.state == "present":
            return self._present(resource)
        elif self.state == "absent":
            return self.resource_absent(resource)

    def _present(self, resource):
        scope_uris = self.data.pop("scopeUris", None)
        self._replace_network_name_by_uri(self.data)
        result = self.resource_present(resource, self.RESOURCE_FACT_NAME)
        if scope_uris is not None:
            result = self.resource_scopes_set(result, self.RESOURCE_FACT_NAME, scope_uris)
        return result

    def _get_ethernet_network_by_name(self, name):
        result = self.oneview_client.ethernet_networks.get_by("name", name)
        return result[0] if result else None

    def _get_network_uri(self, network_name_or_uri):
        if network_name_or_uri.startswith("/rest/ethernet-networks"):
            return network_name_or_uri
        else:
            enet_network = self._get_ethernet_network_by_name(network_name_or_uri)
            if enet_network:
                return enet_network["uri"]
            else:
                raise OneViewModuleResourceNotFound(self.MSG_ETHERNET_NETWORK_NOT_FOUND + network_name_or_uri)

    def _replace_network_name_by_uri(self, data):
        if "networkUris" in data:
            data["networkUris"] = [self._get_network_uri(x) for x in data["networkUris"]]


def main():
    NetworkSetModule().run()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2012, Jim Richardson <weaselkeeper@gmail.com>
# Copyright (c) 2019, Bernd Arnold <wopfel@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pushover
short_description: Send notifications through U(https://pushover.net)
description:
  - Send notifications through pushover to subscriber list of devices and email addresses. Requires pushover app on devices.
notes:
  - You need a pushover.net account to use this module. But no account is required to receive messages.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  msg:
    type: str
    description:
      - What message you wish to send.
    required: true
  app_token:
    type: str
    description:
      - Pushover issued token identifying your pushover app.
    required: true
  user_key:
    type: str
    description:
      - Pushover issued authentication key for your user.
    required: true
  title:
    type: str
    description:
      - Message title.
  pri:
    type: str
    description:
      - Message priority (see U(https://pushover.net) for details).
    default: '0'
    choices: ['-2', '-1', '0', '1', '2']
  device:
    type: str
    description:
      - A device the message should be sent to. Multiple devices can be specified, separated by a comma.
    version_added: 1.2.0

author:
  - "Jim Richardson (@weaselkeeper)"
  - "Bernd Arnold (@wopfel)"
"""

EXAMPLES = r"""
- name: Send notifications via pushover.net
  community.general.pushover:
    msg: '{{ inventory_hostname }} is acting strange ...'
    app_token: wxfdksl
    user_key: baa5fe97f2c5ab3ca8f0bb59
  delegate_to: localhost

- name: Send notifications via pushover.net
  community.general.pushover:
    title: 'Alert!'
    msg: '{{ inventory_hostname }} has exploded in flames, It is now time to panic'
    pri: 1
    app_token: wxfdksl
    user_key: baa5fe97f2c5ab3ca8f0bb59
  delegate_to: localhost

- name: Send notifications via pushover.net to a specific device
  community.general.pushover:
    msg: '{{ inventory_hostname }} has been lost somewhere'
    app_token: wxfdksl
    user_key: baa5fe97f2c5ab3ca8f0bb59
    device: admins-iPhone
  delegate_to: localhost
"""

from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


class Pushover:
    """Instantiates a pushover object, use it to send notifications"""

    base_uri = "https://api.pushover.net"

    def __init__(self, module, user, token):
        self.module = module
        self.user = user
        self.token = token

    def run(self, priority, msg, title, device):
        """Do, whatever it is, we do."""

        url = f"{self.base_uri}/1/messages.json"

        # parse config
        options = dict(user=self.user, token=self.token, priority=priority, message=msg)

        if title is not None:
            options = dict(options, title=title)

        if device is not None:
            options = dict(options, device=device)

        data = urlencode(options)

        headers = {"Content-type": "application/x-www-form-urlencoded"}
        r, info = fetch_url(self.module, url, method="POST", data=data, headers=headers)
        if info["status"] != 200:
            raise Exception(info)

        return r.read()


def main():
    module = AnsibleModule(
        argument_spec=dict(
            title=dict(type="str"),
            msg=dict(required=True),
            app_token=dict(required=True, no_log=True),
            user_key=dict(required=True, no_log=True),
            pri=dict(default="0", choices=["-2", "-1", "0", "1", "2"]),
            device=dict(type="str"),
        ),
    )

    msg_object = Pushover(module, module.params["user_key"], module.params["app_token"])
    try:
        response = msg_object.run(
            module.params["pri"], module.params["msg"], module.params["title"], module.params["device"]
        )
    except Exception:
        module.fail_json(msg="Unable to send msg via pushover")

    module.exit_json(msg=f"message sent successfully: {response}", changed=False)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2026, Alexei Znamensky (russoz) <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: snap_connect
short_description: Manages snap interface connections
version_added: "12.6.0"
description:
  - Manages connections between snap plugs and slots.
  - Snaps run in a sandbox and need explicit interface connections to access system resources
    or communicate with other snaps.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  state:
    description:
      - Desired state of the connection.
    type: str
    choices: [absent, present]
    default: present
  plug:
    description:
      - The plug endpoint in the format C(<snap>:<plug>).
    type: str
    required: true
  slot:
    description:
      - The slot endpoint in the format C(<snap>:<slot>) or C(:<slot>) for system slots.
      - If not specified, snapd will attempt to find a matching slot automatically.
    type: str
notes:
  - Privileged operations require root privileges.
author:
  - Alexei Znamensky (@russoz) <russoz@gmail.com>
seealso:
  - module: community.general.snap
  - module: community.general.snap_alias
"""

EXAMPLES = r"""
- name: Connect firefox camera plug to system camera slot
  community.general.snap_connect:
    plug: firefox:camera
    slot: ":camera"

- name: Connect snap plug (slot resolved automatically by snapd)
  community.general.snap_connect:
    plug: my-app:network

- name: Disconnect a specific connection
  community.general.snap_connect:
    plug: firefox:camera
    slot: ":camera"
    state: absent
"""

RETURN = r"""
snap_connections:
  description: The list of snap connections after execution.
  type: list
  elements: dict
  returned: always
  contains:
    interface:
      description: The interface name.
      type: str
    plug:
      description: The plug endpoint.
      type: str
    slot:
      description: The slot endpoint.
      type: str
version:
  description: Versions of snap components as reported by C(snap version).
  type: dict
  returned: always
"""

import re

from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper
from ansible_collections.community.general.plugins.module_utils._snap import get_version, snap_runner


class SnapConnect(StateModuleHelper):
    _RE_CONNECTIONS = re.compile(r"^(?P<interface>\S+)\s+(?P<plug>\S+)\s+(?P<slot>\S+)\s+.*$")

    module = dict(
        argument_spec={
            "state": dict(type="str", choices=["absent", "present"], default="present"),
            "plug": dict(type="str", required=True),
            "slot": dict(type="str"),
        },
        supports_check_mode=True,
    )

    def __init_module__(self):
        self.runner = snap_runner(self.module)
        self.vars.version = get_version(self.runner)
        self.vars.set("snap_connections", self._get_connections(), change=True, diff=True)

    def __quit_module__(self):
        self.vars.snap_connections = self._get_connections()

    def _get_connections(self):
        def process(rc, out, err):
            if rc != 0:
                return []
            connections = []
            for line in out.splitlines()[1:]:
                match = self._RE_CONNECTIONS.match(line.strip())
                if match:
                    connections.append(match.groupdict())
            return connections

        with self.runner("_connections", output_process=process) as ctx:
            return ctx.run()

    def _is_connected(self):
        plug = self.vars.plug
        slot = self.vars.slot
        return any(
            conn["slot"] != "-" and conn["plug"] == plug and (slot is None or conn["slot"] == slot)
            for conn in self.vars.snap_connections
        )

    def state_present(self):
        if not self._is_connected():
            self.changed = True
            if self.check_mode:
                return
            with self.runner("_connect slot") as ctx:
                rc, dummy, err = ctx.run(_connect=self.vars.plug, slot=self.vars.slot)
            if rc != 0:
                self.do_raise(msg=f"snap connect failed: {err}")

    def state_absent(self):
        if self._is_connected():
            self.changed = True
            if self.check_mode:
                return
            with self.runner("_disconnect slot") as ctx:
                rc, dummy, err = ctx.run(_disconnect=self.vars.plug, slot=self.vars.slot)
            if rc != 0:
                self.do_raise(msg=f"snap disconnect failed: {err}")


def main():
    SnapConnect.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2014, Kim Nørgaard
# Written by Kim Nørgaard <jasen@jasen.dk>
# Based on pkgng module written by bleader <bleader@ratonland.org>
# that was based on pkgin module written by Shaun Zinck <shaun.zinck at gmail.com>
# that was based on pacman module written by Afterburn <https://github.com/afterburn>
# that was based on apt module written by Matthew Williams <matthew@flowroute.com>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: slackpkg
short_description: Package manager for Slackware >= 12.2
description:
  - Manage binary packages for Slackware using C(slackpkg) which is available in versions after 12.2.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of package to install/remove.
    required: true
    type: list
    elements: str
    aliases: [pkg]

  state:
    description:
      - State of the package, you can use V(installed) as an alias for V(present) and V(removed) as one for V(absent).
    choices: ['present', 'absent', 'latest', 'installed', 'removed']
    default: present
    type: str

  update_cache:
    description:
      - Update the package database first.
    default: false
    type: bool

author: Kim Nørgaard (@KimNorgaard)
requirements: ["Slackware >= 12.2"]
"""

EXAMPLES = r"""
- name: Install package foo
  community.general.slackpkg:
    name: foo
    state: present

- name: Remove packages foo and bar
  community.general.slackpkg:
    name: foo,bar
    state: absent

- name: Make sure that it is the most updated package
  community.general.slackpkg:
    name: foo
    state: latest
"""

import os
import platform
import re

from ansible.module_utils.basic import AnsibleModule


def query_package(name):
    machine = platform.machine()
    # Exception for kernel-headers package on x86_64
    if name == "kernel-headers" and machine == "x86_64":
        machine = "x86"
    pattern = re.compile(f"^{re.escape(name)}-[^-]+-({re.escape(machine)}|noarch|fw)-[^-]+$")
    packages = [f for f in os.listdir("/var/log/packages") if pattern.match(f)]

    return bool(packages)


def remove_packages(module, slackpkg_path, packages):
    remove_c = 0
    # Using a for loop in case of error, we can report the package that failed
    for package in packages:
        # Query the package first, to see if we even need to remove
        if not query_package(package):
            continue

        if not module.check_mode:
            rc, out, err = module.run_command([slackpkg_path, "-default_answer=y", "-batch=on", "remove", package])

        if not module.check_mode and query_package(package):
            module.fail_json(msg=f"failed to remove {package}: {out}")

        remove_c += 1

    if remove_c > 0:
        module.exit_json(changed=True, msg=f"removed {remove_c} package(s)")

    module.exit_json(changed=False, msg="package(s) already absent")


def install_packages(module, slackpkg_path, packages):
    install_c = 0

    for package in packages:
        if query_package(package):
            continue

        if not module.check_mode:
            rc, out, err = module.run_command([slackpkg_path, "-default_answer=y", "-batch=on", "install", package])

        if not module.check_mode and not query_package(package):
            module.fail_json(msg=f"failed to install {package}: {out}", stderr=err)

        install_c += 1

    if install_c > 0:
        module.exit_json(changed=True, msg=f"present {install_c} package(s)")

    module.exit_json(changed=False, msg="package(s) already present")


def upgrade_packages(module, slackpkg_path, packages):
    install_c = 0

    for package in packages:
        if not module.check_mode:
            rc, out, err = module.run_command([slackpkg_path, "-default_answer=y", "-batch=on", "upgrade", package])

        if not module.check_mode and not query_package(package):
            module.fail_json(msg=f"failed to install {package}: {out}", stderr=err)

        install_c += 1

    if install_c > 0:
        module.exit_json(changed=True, msg=f"present {install_c} package(s)")

    module.exit_json(changed=False, msg="package(s) already present")


def update_cache(module, slackpkg_path):
    rc, out, err = module.run_command([slackpkg_path, "-batch=on", "update"])
    if rc != 0:
        module.fail_json(msg="Could not update package cache")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(default="present", choices=["installed", "removed", "absent", "present", "latest"]),
            name=dict(aliases=["pkg"], required=True, type="list", elements="str"),
            update_cache=dict(default=False, type="bool"),
        ),
        supports_check_mode=True,
    )

    slackpkg_path = module.get_bin_path("slackpkg", True)

    p = module.params

    pkgs = p["name"]

    if p["update_cache"]:
        update_cache(module, slackpkg_path)

    if p["state"] == "latest":
        upgrade_packages(module, slackpkg_path, pkgs)

    elif p["state"] in ["present", "installed"]:
        install_packages(module, slackpkg_path, pkgs)

    elif p["state"] in ["removed", "absent"]:
        remove_packages(module, slackpkg_path, pkgs)


if __name__ == "__main__":
    main()
